/*******************************************************************************
*   Project BUTTON                                                             *
*   http://space.tin.it/computer/giparrel, joparrel@tin.it                     *
*   Copyright  1998. All Rights Reserved.                                     *
*                                                                              *
*   SUBSYSTEM:    button.apx Application                                       *
*   FILE:         apxprev.cpp                                                  *
*   AUTHOR:       Jo Parrello.                                                 *
*                                                                              *
*   OVERVIEW                                                                   *
*   ~~~~~~~~----                                                               *
*   Source file for implementation of Print Preview.                           *
*                                                                              *
*   Revision:   1.0  (May 15, 1998).                                           *
*                                                                              *
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF          *
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO        *
*   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A             *
*   PARTICULAR PURPOSE.                                                        *
*******************************************************************************/
#include <owl/pch.h>

#include <owl/buttonga.h>
#include <owl/textgadg.h>
#include <stdio.h>

#include "apxprev.h"
#include "apxprev.rh"
#include "jpflbute.h"
#include "jpfltdiv.h"

//{{TApxPreviewWin Implementation}}

DEFINE_RESPONSE_TABLE1(TApxPreviewWin, TDecoratedFrame)
  EV_COMMAND_ENABLE(APX_PPR_PREVIOUS, PPR_PreviousEnable),
  EV_COMMAND_ENABLE(APX_PPR_NEXT, PPR_NextEnable),
  EV_COMMAND(APX_PPR_PREVIOUS, PPR_Previous),
  EV_COMMAND(APX_PPR_NEXT, PPR_Next),
  EV_COMMAND(APX_PPR_ONEUP, PPR_OneUp),
  EV_COMMAND_ENABLE(APX_PPR_ONEUP, PPR_OneUpEnable),
  EV_COMMAND_ENABLE(APX_PPR_TWOUP, PPR_TwoUpEnable),
  EV_COMMAND(APX_PPR_TWOUP, PPR_TwoUp),
  EV_COMMAND(APX_PPR_DONE, PPR_Done),
  EV_COMMAND(CM_FILEPRINT, CmPrint),
  EV_COMMAND_ENABLE(CM_FILEPRINT, CmPrintEnable),
//{{TApxPreviewWinRSP_TBL_BEGIN}}
  EV_WM_CLOSE,
  EV_COMMAND_ENABLE(CM_FULLSCREEN, CeFullscreen),
  EV_COMMAND(CM_FULLSCREEN, CmFullscreen),
//{{TApxPreviewWinRSP_TBL_END}}
END_RESPONSE_TABLE;


HBITMAP WINAPI CreateMenuBitmap(bool IsRadioChecked, uint factor)
{
    // Create a DC compatible with the desktop window's DC.
    HWND hwndDesktop = ::GetDesktopWindow();
    HDC hdcDesktop = ::GetDC(hwndDesktop);
    if (!hdcDesktop)
    	return NULL;
    HDC hdcMem = ::CreateCompatibleDC(hdcDesktop);
    if (!hdcMem)
    	return NULL;

    // Determine the required bitmap size.
    DWORD dwExt = ::GetMenuCheckMarkDimensions();
    SIZE size = { LOWORD(dwExt), HIWORD(dwExt) };

    // Create a monochrome bitmap and select it.
    HBITMAP hbm = ::CreateBitmap(size.cx, size.cy, 1, 1, NULL);
    if (!hbm)
    	return NULL;
    HBITMAP hbmOld = (HBITMAP) ::SelectObject(hdcMem, hbm);
    if (!hbmOld)
    	return NULL;

    // Erase the background and call the drawing function.
    if (!::PatBlt(hdcMem, 0, 0, size.cx, size.cy, WHITENESS))
    	return NULL;

    if (IsRadioChecked)
        {
        uint centerx = size.cx / 2;
        uint centery = size.cy / 2;
        if (!::SelectObject(hdcMem, GetStockObject(BLACK_BRUSH)))
          return NULL;
        if (::Ellipse(hdcMem, centerx - factor, centery - factor, centerx + factor, centery + factor)==0)
          return NULL;
        }

    // Clean up.
    ::SelectObject(hdcMem, hbmOld);
    ::DeleteDC(hdcMem);
    ::ReleaseDC(hwndDesktop, hdcDesktop);
    return hbm;
}

//
//
//
void WINAPI SetMenuRadioItem(HMENU Handle, uint first, uint last, uint factor)
{
    HBITMAP hbmChecked = CreateMenuBitmap(true,factor);
    if (!hbmChecked)
    	return;
    HBITMAP hbmUnchecked = CreateMenuBitmap(false,factor);
    if (!hbmUnchecked)
    	return;

    // Set the checkmark bitmaps for each menu item.
    for (uint i = first; i <= last; i++)
        ::SetMenuItemBitmaps(Handle, i, MF_BYCOMMAND, hbmUnchecked, hbmChecked);
}


TApxPreviewWin::TApxPreviewWin(TWindow* parentWindow, TPrinter* printer,
                               TWindow* currWindow, const char far* title,
                               TLayoutWindow* client)
:
  TDecoratedFrame(parentWindow, title, client, true)
{
  CurrWindow = currWindow;
  Printer = printer;
  Client = client;
  PageList = 0;
  Page1 = 0;
  Page2 = 0;
  FromPage = 1;
  ToPage = 1;
  IsFullScreen = false;

  TPrintDialog::TData& data = Printer->GetSetup();
  PrnDC = new TPrintDC(data.GetDriverName(),
                       data.GetDeviceName(),
                       data.GetOutputName(),
                       data.GetDevMode());

  PrintExtent = new TSize(PrnDC->GetDeviceCaps(HORZRES), PrnDC->GetDeviceCaps(VERTRES));
  Printout = new TApxPrintout(Printer, "Print Preview", currWindow, true);

  // Create default toolbar New and associate toolbar buttons with commands.
  //
  PreviewSpeedBar = new TControlBar(this, TControlBar::Horizontal);
  JPFlatButtonTextGadget::JPDisplayType dtype = JPFlatButtonTextGadget::BitmapText;
  JPFlatButtonTextGadget::JPTextPos     dpos  = JPFlatButtonTextGadget::Bottom;
  PreviewSpeedBar->Insert(*new JPFlatButtonTextGadget(NULL, dtype, dpos, 5, 1, APX_PPR_PREVIOUS, APX_PPR_PREVIOUS, TButtonGadget::Command, true));
  PreviewSpeedBar->Insert(*new JPFlatButtonTextGadget(NULL, dtype, dpos, 5, 1, APX_PPR_NEXT, APX_PPR_NEXT, TButtonGadget::Command, true));
  PreviewSpeedBar->Insert(*new JPFlatDivGadget(0));
  PreviewSpeedBar->Insert(*new JPFlatButtonTextGadget(NULL, dtype, dpos, 5, 1, APX_PPR_ONEUP, APX_PPR_ONEUP, TButtonGadget::Exclusive, true, TButtonGadget::Down));
  PreviewSpeedBar->Insert(*new JPFlatButtonTextGadget(NULL, dtype, dpos, 5, 1, APX_PPR_TWOUP, APX_PPR_TWOUP, TButtonGadget::Exclusive, true));
  PreviewSpeedBar->Insert(*new JPFlatDivGadget(0));
  PreviewSpeedBar->Insert(*new TTextGadget(APX_PPR_CURRPAGE, TGadget::Recessed, TTextGadget::Left, 10, "Page 1"));
  PreviewSpeedBar->Insert(*new JPFlatDivGadget(0));
  PreviewSpeedBar->Insert(*new JPFlatButtonTextGadget(NULL, dtype, dpos, 5, 1, CM_FULLSCREEN, CM_FULLSCREEN, TButtonGadget::NonExclusive, true, TButtonGadget::Up));
  PreviewSpeedBar->Insert(*new JPFlatDivGadget(0));
  PreviewSpeedBar->Insert(*new JPFlatButtonTextGadget(NULL, dtype, dpos, 5, 1, CM_FILEPRINT, CM_FILEPRINT, TButtonGadget::Command, true));
  PreviewSpeedBar->Insert(*new JPFlatDivGadget(0));
  PreviewSpeedBar->Insert(*new JPFlatButtonTextGadget(NULL, dtype, dpos, 5, 1, APX_PPR_DONE, APX_PPR_DONE, TButtonGadget::Command, true));
  PreviewSpeedBar->SetCaption("Preview Toolbar");
  PreviewSpeedBar->SetHintMode(TGadgetWindow::EnterHints);
  Insert(*PreviewSpeedBar, TDecoratedFrame::Top);

  sb = new TStatusBar(this, TGadget::Recessed,
                                  TStatusBar::CapsLock        |
                                  TStatusBar::NumLock         |
                                  TStatusBar::ScrollLock);
  sb->SetCaption("Preview Status Bar");
  sb->SetHintMode(TGadgetWindow::EnterHints);
  sb->SetWideHints(false);
  Insert(*sb, TDecoratedFrame::Bottom);

  Attr.Style &= ~WS_VISIBLE;
  SetAcceleratorTable(IDM_PRINTPREVIEW);
  Client->SetBkgndColor(TColor::Sys3dShadow);
}


TApxPreviewWin::~TApxPreviewWin()
{
  delete Page1;
  delete Page2;
  delete PrnDC;
  delete PrintExtent;
  delete Printout;
  delete PageList;
}


void TApxPreviewWin::SetupWindow()
{
  TDecoratedFrame::SetupWindow();

  TAPointer<char> captionText = new char[256];
  PageList = new TCollection<TPreviewPage>(5,5);
  // Set the caption of the preview window based on that of the Main Window.
  //
  GetApplication()->GetMainWindow()->GetWindowText(captionText, 256);
  strcat(captionText, " (Preview)");
  SetCaption(captionText);

  Client->Invalidate(true);
  Client->UpdateWindow();

  // Set the icons of the preview window.
  //
  SetIcon(GetApplication(), IDI_MDIAPPLICATION);
  SetIconSm(GetApplication(), IDI_MDIAPPLICATION);

  TPrintDialog::TData& data = Printer->GetSetup();
  Page1 = new TPreviewPage(Client, *Printout, *PrnDC, *PrintExtent, 1);
  Page1->SetPageNumber(1);
  data.MaxPage = 1;

  Page2 = 0;

  TLayoutMetrics metrics1;

  metrics1.X.Set(lmLeft, lmRightOf, lmParent, lmLeft, 30);
  metrics1.Y.Set(lmTop, lmBelow, lmParent, lmTop, 15);

  // Determine major axis of preview page, have that follow parent size.
  // Make minor axis a percentage (aspect ratio) of the page's major axis
  //
  TRect r = Client->GetClientRect();
  long ratio;

  if (PrintExtent->cx > PrintExtent->cy)
    ratio = ((long)PrintExtent->cy * 100) / PrintExtent->cx;
  else
    ratio = ((long)PrintExtent->cx * 100) / PrintExtent->cy;

  bool xMajor = ((r.Width() * ratio) / 100) > r.Height();
  if (xMajor){
    metrics1.Height.Set(lmBottom, lmAbove, lmParent, lmBottom, 15);
    metrics1.Width.PercentOf(Page1, (int)((long)PrintExtent->cx * 95 / PrintExtent->cy), lmHeight);
  }
  else {
    metrics1.Height.PercentOf(Page1, (int)((long)PrintExtent->cy * 95 / PrintExtent->cx), lmWidth);
    metrics1.Width.Set(lmRight, lmLeftOf, lmParent, lmRight, 15);
  }

  Page1->Create();

  Client->SetChildLayoutMetrics(*Page1, metrics1);
  Client->Layout();
  AssignMenu(IDM_PREVIEWMENU);
  SetMenuRadioItem(GetMenu(), APX_PPR_ONEUP, APX_PPR_TWOUP, 2);
  PPR_OneUp();
}


void TApxPreviewWin::SpeedBarState()
{
  // Update the page count.
  //
  TTextGadget* cpGadget = TYPESAFE_DOWNCAST(PreviewSpeedBar->GadgetWithId(APX_PPR_CURRPAGE), TTextGadget);
  if (cpGadget) {
    TAPointer<char> buffer = new char[32];

    if (Page2 && FromPage != ToPage)
      sprintf(buffer, "Page %d - %d", FromPage, ToPage);
    else
      sprintf(buffer, "Page %d", FromPage);
    cpGadget->SetText(buffer);
  }
}


void TApxPreviewWin::PPR_PreviousEnable(TCommandEnabler& tce)
{
  // Only have previous on if we're not at the first page.
  //
  tce.Enable(FromPage != 1);
}


void TApxPreviewWin::PPR_NextEnable(TCommandEnabler& tce)
{
  // Only have next on if we're not at the last page.
  //
  TPrintDialog::TData& printerData = Printer->GetSetup();
  tce.Enable(ToPage != printerData.MaxPage);
}


void TApxPreviewWin::PPR_Previous()
{
  TPrintDialog::TData& printerData = Printer->GetSetup();

  if (FromPage > printerData.MinPage) {
    FromPage--;
    ToPage--;

    Page1->SetPageNumber(FromPage);
    if (Page2)
      Page2->SetPageNumber(ToPage);
  }

  SpeedBarState();
}


void TApxPreviewWin::PPR_Next()
{
  TPrintDialog::TData& printerData = Printer->GetSetup();

  if (ToPage < printerData.MaxPage) {
    FromPage++;
    ToPage++;

    Page1->SetPageNumber(FromPage);
    if (Page2)
      Page2->SetPageNumber(ToPage);
  }

  SpeedBarState();
}


void TApxPreviewWin::PPR_OneUp()
{
  if (Page2) {
    Client->RemoveChildLayoutMetrics(*Page2);
    TRect rect;
    Page2->GetClientRect(rect);
    InvalidateRect(rect, true);

    delete Page2;
    Page2 = 0;

    Client->Layout();

    ToPage = FromPage;

    SpeedBarState();
  }
}


void TApxPreviewWin::PPR_OneUpEnable(TCommandEnabler& tce)
{
  // Two up is only available for portrait mode.
  //
  tce.Enable(true);
  tce.SetCheck(Page2 == NULL);
}

void TApxPreviewWin::PPR_TwoUpEnable(TCommandEnabler& tce)
{
  // Two up is only available for portrait mode.
  //
  tce.Enable(PrintExtent->cx <= PrintExtent->cy);
  tce.SetCheck(Page2 != NULL);
}


void TApxPreviewWin::PPR_TwoUp()
{
  if (!Page2) {
    Page2 = new TPreviewPage(Client, *Printout, *PrnDC, *PrintExtent, PageNumber + 1);
    Page2->Create();

    TLayoutMetrics metrics2;

    metrics2.X.Set(lmLeft, lmRightOf, Page1, lmRight, 30);
    metrics2.Y.SameAs(Page1, lmTop);

    // Assume portrait
    //
    metrics2.Width.SameAs(Page1, lmWidth);
    metrics2.Height.SameAs(Page1, lmBottom);

    Client->SetChildLayoutMetrics(*Page2, metrics2);
    Client->Layout();

    TPrintDialog::TData& printerData = Printer->GetSetup();

    // Page 2 is the next page.  If the next page is outside of our range then
    // set the first page back one and the 2nd page is the current page. If the
    // document is only 1 page long then the 2nd page is empty.
    //
    if (FromPage == printerData.MaxPage) {
      if (FromPage > 1) {
        FromPage--;
        ToPage = FromPage + 1;
        Page1->SetPageNumber(FromPage);
        Page2->SetPageNumber(ToPage);
      }
      else
        Page2->SetPageNumber(0);
    }
    else {
      ToPage = FromPage + 1;
      Page2->SetPageNumber(ToPage);
    }

    SpeedBarState();
  }
}


void TApxPreviewWin::PPR_Done()
{
  // Don't call the base class EvClose; we do not want TApxPreviewWin to be destructed.
  //
  if (IsFullScreen)
     CmFullscreen();
  else
     GetApplication()->EndModal(IDCANCEL);
}


void TApxPreviewWin::EvClose()
{
  // Don't call the base class EvClose; we do not want TApxPreviewWin to be destructed.
  //
  GetApplication()->EndModal(IDCANCEL);
}


void TApxPreviewWin::CmPrint()
{
  TWindow* client = GetApplication()->GetMainWindow()->GetClientWindow();

  if (client)
    client->SendMessage(WM_COMMAND, CM_FILEPRINT, 0);
}


void TApxPreviewWin::CmPrintEnable(TCommandEnabler& tce)
{
  tce.Enable(true);
}


void TApxPreviewWin::CeFullscreen(TCommandEnabler &tce)
{
  tce.Enable(true);
  tce.SetCheck(IsFullScreen);
}


void TApxPreviewWin::CmFullscreen()
{
  long newLong;
  // INSERT>> Your code here.
  if (IsFullScreen)
  	  {
     IsFullScreen = false;
     // restore the window's style
     SetWindowLong(GWL_STYLE, mainLong);
     // restore the menu
     SetMenu(hMenu);
     // re-show the status bar
     Insert(*sb, TDecoratedFrame::Bottom);
     sb->ShowWindow(SW_NORMAL);
     // restore the window's position
     SetWindowPlacement(&mainPos);
     Invalidate();
     UpdateWindow();
     }
  else
  	  {
     IsFullScreen = true;
     // get the current window's position
     mainPos.length = sizeof(WINDOWPLACEMENT);  // to solve a horrific bug on TWindow class
     GetWindowPlacement(&mainPos);
     // get the current window's style
     mainLong = (long) GetWindowLong(GWL_STYLE);
     newLong = mainLong;
     newLong &= ~(WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX);
     SetWindowLong(GWL_STYLE, newLong);
     Invalidate();
     UpdateWindow();
     // hides the status bar
     sb->ShowWindow(SW_HIDE);
     Insert(*sb, TDecoratedFrame::None);
     // hides the menu
     hMenu = GetMenu();
     SetMenu(NULL);
     // maximizes the window
     if (mainPos.showCmd != SW_SHOWMAXIMIZED)
        ShowWindow(SW_SHOWMAXIMIZED);
     }
  Invalidate();
  UpdateWindow();
}

int  TApxPreviewWin::GetPageCount()
{
return PageList->GetCount();
}

