/*******************************************************************************
*   Project BUTTON                                                             *
*   http://space.tin.it/computer/giparrel, joparrel@tin.it                     *
*   Copyright  1998. All Rights Reserved.                                     *
*                                                                              *
*   SUBSYSTEM:    BUTTON Application                                           *
*   FILE:         buttonap.cpp                                                 *
*   AUTHOR:       Jo Parrello.                                                 *
*                                                                              *
*   OVERVIEW                                                                   *
*   ~~~~~~~~----                                                               *
*   Source file for implementation of TBUTTONApp (TApplication).               *
*                                                                              *
*   Revision:   1.0  (May 15, 1998).                                           *
*                                                                              *
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF          *
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO        *
*   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A             *
*   PARTICULAR PURPOSE.                                                        *
*******************************************************************************/

#include <owl/pch.h>

#include <owl/buttonga.h>
#include <owl/docmanag.h>
#include <owl/filedoc.h>
#include <owl/docking.h>
#include <owl/controlg.h>

#include <classlib/cmdline.h>

#include "buttonap.h"
#include "buttonmd.h"
#include "buttonm1.h"
#include "buttonev.h"
#include "jpcstbar.h"
#include "jpflmenu.h"
#include "buttonad.h"                        // Definition of about dialog.

const char HelpFileName[] = "BUTTON.hlp";

/********************************************************/
/*          ADDED TO SUPPORT MULTIPLE TOOLBARS          */
/********************************************************/
const char IniFile[]      = ".\\BUTTON.ini";

typedef struct tagTOOLBARDATA
{
    int  GadgetId;
    int  ToolbarPos;
} TOOLBARDATA;
typedef TOOLBARDATA NEAR* NPTOOLBARDATA;
typedef TOOLBARDATA FAR*  LPTOOLBARDATA;

/****     DEFAULT TOOLBAR DATA  (LOADED WHEN THERE ARE NO DATA FROM .INI FILE    *****/
const TOOLBARDATA GadgetList[] =   { //   Button Id,       Toolbar Index (stored in ToolbarList).
                                   { CM_MDIFILENEW,        0 },
                                   { CM_MDIFILEOPEN,       0 },
                                   { CM_FILESAVE,          0 },
                                   { 0,                    0 },
                                   { CM_EDITCUT,           0 },
                                   { CM_EDITCOPY,          0 },
                                   { CM_EDITPASTE,         0 },
                                   { 0,                    0 },
                                   { CM_EDITUNDO,          0 },
                                   { 0,                    0 },
                                   { CM_EDITFIND,          0 },
                                   { CM_EDITFINDNEXT,      0 },
                                   { 0,                    0 },
                                   { CM_FILEPRINT,         0 },
                                   { CM_FILEPRINTPREVIEW,  0 },
                                   { 0,                    0 },
                                   { CM_HELPCONTENTS,      0 },
                                   { 0,                    0 },
                                   { ID_COMBOBOX,          0 },
                                   { CM_TOOLBARCUSTOMIZE,  1 },
                                   { 0,                    1 },
                                   { CM_TOOLBARBITMAP,     1 },
                                   { CM_TOOLBARTEXT,       1 },
                                   { CM_TOOLBARBITMAPTEXT, 1 }
                                   };

//{{TBUTTONApp Implementation}}



//{{DOC_VIEW}}
DEFINE_DOC_TEMPLATE_CLASS(TFileDocument, TBUTTONEditView, DocType1);
//{{DOC_VIEW_END}}

//{{DOC_MANAGER}}
DocType1 __dvt1("All Files", "*.*", 0, "txt", dtAutoDelete | dtUpdateDir | dtOverwritePrompt);
//{{DOC_MANAGER_END}}


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE2(TBUTTONApp, TRecentFiles, TApplication)
//{{TBUTTONAppRSP_TBL_BEGIN}}
  EV_OWLVIEW(dnCreate, EvNewView),
  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_HELPCONTENTS, CmHelpContents),
  EV_COMMAND(CM_HELPUSING, CmHelpUsing),
  EV_WM_DROPFILES,
  EV_WM_WININICHANGE,
  EV_OWLDOCUMENT(dnCreate, EvOwlDocument),
  EV_OWLDOCUMENT(dnRename, EvOwlDocument),
  EV_REGISTERED(MruFileMessage, CmFileSelected),
/*******    ADDED TO SUPPORT MULTIPLE TOOLBARS            */
  EV_COMMAND_ENABLE(CM_TOOLBARBITMAP, CeToolbarbitmap),
  EV_COMMAND_ENABLE(CM_TOOLBARBITMAPTEXT, CeToolbarbitmaptext),
  EV_COMMAND_ENABLE(CM_TOOLBARBOTTOM, CeToolbarbottom),
  EV_COMMAND_ENABLE(CM_TOOLBARLEFT, CeToolbarleft),
  EV_COMMAND_ENABLE(CM_TOOLBARRIGHT, CeToolbarright),
  EV_COMMAND_ENABLE(CM_TOOLBARTEXT, CeToolbartext),
  EV_COMMAND_ENABLE(CM_TOOLBARTOP, CeToolbartop),
  EV_COMMAND_ENABLE(CM_ONETEXTROW, CeOneTextRow),
  EV_COMMAND_ENABLE(CM_TWOTEXTROW, CeTwoTextRow),
  EV_COMMAND(CM_TOOLBARBITMAP, CmToolbarbitmap),
  EV_COMMAND(CM_TOOLBARBITMAPTEXT, CmToolbarbitmaptext),
  EV_COMMAND(CM_TOOLBARBOTTOM, CmToolbarbottom),
  EV_COMMAND(CM_TOOLBARLEFT, CmToolbarleft),
  EV_COMMAND(CM_TOOLBARRIGHT, CmToolbarright),
  EV_COMMAND(CM_TOOLBARTEXT, CmToolbartext),
  EV_COMMAND(CM_TOOLBARTOP, CmToolbartop),
  EV_COMMAND(CM_ONETEXTROW, CmOneTextRow),
  EV_COMMAND(CM_TWOTEXTROW, CmTwoTextRow),
  EV_COMMAND(CM_DECREASECHARS, CmDecreasechars),
  EV_COMMAND_ENABLE(CM_DECREASECHARS, CeDecreasechars),
  EV_COMMAND(CM_DEFAULTCHARS, CmDefaultchars),
  EV_COMMAND_ENABLE(CM_DEFAULTCHARS, CeDefaultchars),
  EV_COMMAND(CM_INCREASECHARS, CmIncreasechars),
  EV_COMMAND_ENABLE(CM_INCREASECHARS, CeIncreasechars),
  EV_COMMAND(CM_HOTTRACKING, CmHottracking),
  EV_COMMAND_ENABLE(CM_HOTTRACKING, CeHottracking),
  EV_COMMAND(CM_DIVBUTTONS, CmDivbuttons),
  EV_COMMAND_ENABLE(CM_DIVBUTTONS, CeDivbuttons),
  EV_COMMAND(CM_DIVCOMBO, CmDivcombo),
  EV_COMMAND_ENABLE(CM_DIVCOMBO, CeDivcombo),
  EV_COMMAND(CM_DIVNOTHING, CmDivnothing),
  EV_COMMAND_ENABLE(CM_DIVNOTHING, CeDivnothing),
  EV_COMMAND(CM_DIVENDOFLINE, CmDivendofline),
  EV_COMMAND_ENABLE(CM_DIVENDOFLINE, CeDivendofline),
  EV_COMMAND(CM_DIVVERTBAR, CmDivvertbar),
  EV_COMMAND_ENABLE(CM_DIVVERTBAR, CeDivvertbar),
  EV_COMMAND(CM_TOOLBARCUSTOMIZE, CmToolbarcustomize),
  EV_COMMAND_ENABLE(CM_TOOLBARCUSTOMIZE, CeToolbarcustomize),
  EV_COMMAND_ENABLE(ID_BUTTONMENU, CeButtonMenu),
//{{TBUTTONAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
//
TBUTTONApp::TBUTTONApp() : TApplication("BUTTON"), TRecentFiles(IniFile, 10)
, ToolbarList(5, 0, 5)   /******      TOOLBAR LIST ARRAY        *****/
{
  HelpState = false;
  ContextHelp = false;
  HelpCursor = 0;

  Printer = 0;
  Printing = 0;

  SetDocManager(new TDocManager(dmMDI, this));

  // INSERT>> Your constructor code here.
}


TBUTTONApp::~TBUTTONApp()
{
  delete Printer;
  // INSERT>> Your destructor code here.

/*********   DELETE ALL TOOLBARS FROM TOOLBARLIST      ******/
/**   Note : don't remove these lines, you can cause a GPF error!!     ***/
  while (ToolbarList.GetItemsInContainer() > 0)
      ToolbarList.RemoveEntry(0);
}


bool TBUTTONApp::CanClose()
{
  bool result = TApplication::CanClose();

/*****       STORE TOOLBAR PREFERENCES ON PROFILE FILE          */
  if (result)
    StoreToolbarDataInProfile();

  if (result && HelpState)
    GetMainWindow()->WinHelp(HelpFileName, HELP_QUIT, 0);

  return result;
}

void TBUTTONApp::SetupSpeedBar(TDecoratedMDIFrame* frame, TGadgetWindow::TTileDirection direction, TFont* font)
{
  ApxHarbor = new THarbor(*frame);

  /*      STEP 1 :  CREATE YOUR OWN TOOLBARS             */
  /*      Note   :  if you want to put a font param on each toolbar, create a different instance
                    of it.   If you put the same instance, you will cause a GPF error!!!       */
  cb = new TDockableControlBar(frame, direction, font);
  TFont* customFont = new TFont(*font);
  custombar = new TDockableControlBar(frame, direction, customFont);

  /*      STEP 2 :  FILL THE AVAILABLE LIST WITH YOUR GADGETS                                 */
  /*      Note   :  Don't insert separator gadgets. They will be inserted by our functions.   */
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_MDIFILENEW, CM_MDIFILENEW));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_MDIFILEOPEN, CM_MDIFILEOPEN));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_FILESAVE, CM_FILESAVE));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_EDITCUT, CM_EDITCUT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_EDITCOPY, CM_EDITCOPY));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_EDITPASTE, CM_EDITPASTE));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_EDITUNDO, CM_EDITUNDO));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_EDITFIND, CM_EDITFIND));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_EDITFINDNEXT, CM_EDITFINDNEXT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_FILEPRINT, CM_FILEPRINT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_FILEPRINTPREVIEW, CM_FILEPRINTPREVIEW));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_HELPCONTENTS, CM_HELPCONTENTS));
  TMenu tMenu (*this, IDM_FASTPOPMENU);
  GadgetAvailableList->Insert(*new JPFlatMenuButtonTextGadget(tMenu, NULL, dispType, textPos, MaxChars, ToolbarRows, ID_BUTTONMENU, ID_BUTTONMENU));
  flatcombo = new JPFlatComboBox(GetMainWindow(), ID_COMBOBOX, 0,0,150,350, CBS_DROPDOWN, 25);
  flatcombo->SetListBoxWidth(300);
  uint32 style = flatcombo->GetStyle();
  style &= ~CBS_SORT;
  flatcombo->SetStyle(style);
  GadgetAvailableList->Insert(*new TControlGadget(*flatcombo));
  cb->SetCaption("Main Toolbar");
  cb->SetHintMode(TGadgetWindow::EnterHints);
  cb->Attr.Id = IDW_TOOLBAR;
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_TOOLBARCUSTOMIZE, CM_TOOLBARCUSTOMIZE));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_TOOLBARCUSTOMIZE, CM_TOOLBARBITMAP));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_TOOLBARCUSTOMIZE, CM_TOOLBARTEXT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_TOOLBARCUSTOMIZE, CM_TOOLBARBITMAPTEXT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_TOOLBARCUSTOMIZE, CM_TOOLBARLEFT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_TOOLBARCUSTOMIZE, CM_TOOLBARRIGHT));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_TOOLBARCUSTOMIZE, CM_TOOLBARTOP));
  GadgetAvailableList->Insert(*new JPFlatButtonTextGadget(NULL, dispType, textPos, MaxChars, ToolbarRows, CM_TOOLBARCUSTOMIZE, CM_TOOLBARBOTTOM));
  custombar->SetCaption("Customize Toolbar");
  custombar->SetHintMode(TGadgetWindow::EnterHints);

  /*      STEP 3 :  LET TOOLBARLIST KNOWS HOW MANY TOOLBARS IT MUST SUPPORT          */
  ToolbarList.Add(cb);
  ToolbarList.Add(custombar);

  /*      STEP 4 :  PUT EACH TOOLBAR IN HARBOR           */
  ApxHarbor->Insert(*cb, alTop);
  ApxHarbor->Insert(*custombar, alNone);
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Application main window construction & intialization.
//
void TBUTTONApp::InitMainWindow()
{
/********    SET THE INTERNAL VARIABLES TO USE WITH OUR NEW FUNCTIONS          ********/
  dispType = JPFlatButtonTextGadget::BitmapText;
  textPos  = JPFlatButtonTextGadget::Bottom;
  ToolbarRows = 1;
  MaxChars = 4;
  HotTrack = true;
  divGadget = 0;
  divEOL = divVertBar = false;

  if (nCmdShow != SW_HIDE)
      nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;
  MdiClient = new TBUTTONMDIClient(this);
  frame = new TDecoratedMDIFrame(Name, IDM_MDI, *MdiClient, true, this);
  frame->Attr.ExStyle |= WS_EX_ACCEPTFILES;
  frame->SetKeyboardHandling(true);
  nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;
  frame->SetIcon(this, IDI_MDIAPPLICATION);
  frame->SetIconSm(this, IDI_MDIAPPLICATION);
  frame->Attr.AccelTable = IDM_MDI;

/*********      CREATE A "TIMES NEW ROMAN" FONT FOR ALL TOOLBARS      **********/
  LOGFONT MainFontRec;
  MainFontRec.lfHeight = -11;
  MainFontRec.lfWidth = 0;
  MainFontRec.lfEscapement = 0;
  MainFontRec.lfOrientation = 0;
  MainFontRec.lfWeight = FW_NORMAL;
  MainFontRec.lfItalic = 0;
  MainFontRec.lfUnderline = 0;
  MainFontRec.lfStrikeOut = 0;
  MainFontRec.lfCharSet = 0;
  MainFontRec.lfOutPrecision = OUT_STROKE_PRECIS;
  MainFontRec.lfClipPrecision = CLIP_STROKE_PRECIS;
  MainFontRec.lfQuality = DRAFT_QUALITY;
  MainFontRec.lfPitchAndFamily = FF_ROMAN | VARIABLE_PITCH;
  strcpy(MainFontRec.lfFaceName, "Times New Roman");
  toolbarFont = new TFont(&MainFontRec);
  LOGFONT statusbarFont = MainFontRec;
  statusbarFont.lfHeight = -13;
  TFont* sbarFont = new TFont(&statusbarFont);

/*******     CREATE THE AVAILABLE LIST GADGET WINDOW           ***********/
  GadgetAvailableList = new TGadgetWindow();
/*******     FILL THE AVAILABLE LIST AND OTHER INFO      **********/
  SetupSpeedBar(frame, TDockableControlBar::Horizontal, toolbarFont);

  TStatusBar* sb = new TStatusBar(frame, TGadget::Recessed,
                                  TStatusBar::CapsLock        |
                                  TStatusBar::NumLock         |
                                  TStatusBar::ScrollLock, sbarFont);
  sb->Insert(*new TSeparatorGadget(6));
  txtGadg = new TTextGadget(CM_CURRCHARS, TGadget::Recessed, TTextGadget::Left, 5, "Chars = 4");
  sb->Insert(*txtGadg);
  sb->SetWideHints(false);
  frame->Insert(*sb, TDecoratedFrame::Bottom);

  SetMainWindow(frame);
  frame->SetMenuDescr(TMenuDescr(IDM_MDI));
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Application instance initialization.
//
void TBUTTONApp::InitInstance()
{
  TApplication::InitInstance();

  ProcessCmdLine(lpCmdLine);

/***    LOAD THE TOOLBAR DATA FROM THE PROFILE (BEFORE) AND FROM THE ARRAY (AFTER)     *****/
  LoadToolbarDataFromProfile();

/**********   ADDED TO DISPLAY SOME GADGETS' DATA IN COMBOBOX   ************/
  char buf[200];
  int i = 0;
  if (cb)
      {
      for (TGadget* g = cb->FirstGadget(); g; g = cb->NextGadget(*g))
        {
        i++;
        int id = g->GetId();
        wsprintf(buf, "Pos=%.2i   Id=%i", i, id);
        flatcombo->AddString(buf);
        }
      if (i > 0)
         flatcombo->SetSelIndex(0);
      }
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Process command line parameters.
//
void TBUTTONApp::ProcessCmdLine(char * CmdLine)
{
  TCmdLine cmd(CmdLine);
  char buf[_MAX_PATH];

  while (cmd.Kind != TCmdLine::Done) {
    if (cmd.Kind == TCmdLine::Option) {
      if (strnicmp(cmd.Token, "unregister", cmd.TokenLen) == 0) {
        UnRegisterInfo();
        return;
      }
    }
/*******    ADDED TO SUPPORT FILENAMES PASSED IN COMMAND LINE    **********/
    if (cmd.Kind == TCmdLine::Name)
        {
        strncpy(buf, cmd.Token, cmd.TokenLen);
        buf[cmd.TokenLen] = '\0';
        TDocTemplate* tpl = GetDocManager()->MatchTemplate(buf);
        if (tpl)
          GetDocManager()->CreateDoc(tpl, buf);
        }
    cmd.NextToken(true);
  }

  RegisterInfo();
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Register application info.
//
void TBUTTONApp::RegisterInfo()
{
  TAPointer<char> buffer = new char[_MAX_PATH];

  GetModuleFileName(buffer, _MAX_PATH);

  TRegKey(TRegKey::ClassesRoot, "BUTTON.Application\\DefaultIcon").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
  strcat(buffer, ",1");
  TRegKey(TRegKey::ClassesRoot, "BUTTON.Document.1\\DefaultIcon").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
  strcpy(buffer, "BUTTON.Document.1");
  TRegKey(TRegKey::ClassesRoot, ".txt").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Unregister application info.
//
void TBUTTONApp::UnRegisterInfo()
{
  TAPointer<char> buffer = new char[_MAX_PATH];

  GetModuleFileName(buffer, _MAX_PATH);

  TRegKey(TRegKey::ClassesRoot, "BUTTON.Application").DeleteKey("DefaultIcon");
  TRegKey(TRegKey::ClassesRoot, "BUTTON.Document.1").DeleteKey("DefaultIcon");

  TRegKey::ClassesRoot.DeleteKey("BUTTON.Application");
  TRegKey::ClassesRoot.DeleteKey("BUTTON.Document.1");
  TRegKey::ClassesRoot.DeleteKey(".txt");
}



//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Response Table handlers:
//
void TBUTTONApp::EvNewView(TView& view)
{
  TMDIClient* mdiClient = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TMDIClient);
  if (mdiClient) {
    TBUTTONMDIChild* child = new TBUTTONMDIChild(*mdiClient, 0, view.GetWindow());

    if (view.GetViewMenu())
      child->SetMenuDescr(*view.GetViewMenu());

    child->SetIcon(this, IDI_DOC);
    child->SetIconSm(this, IDI_DOC);
    child->Create();
  }
}


void TBUTTONApp::EvCloseView(TView&)
{
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Menu Help Contents command
//
void TBUTTONApp::CmHelpContents()
{
  HelpState = GetMainWindow()->WinHelp(HelpFileName, HELP_CONTENTS, 0);
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~
// Menu Help Using Help command
//
void TBUTTONApp::CmHelpUsing()
{
  HelpState = GetMainWindow()->WinHelp(HelpFileName, HELP_HELPONHELP, 0);
}


//--------------------------------------------------------
// TBUTTONApp
// ~~~~~~~~~~~
// Menu Help About BUTTON command
void TBUTTONApp::CmHelpAbout()
{
  TBUTTONAboutDlg(GetMainWindow()).Execute();
}


void TBUTTONApp::EvDropFiles(TDropInfo drop)
{
  TFileDropletList files;

  int fileCount = drop.DragQueryFileCount();
  for (int i = 0; i < fileCount; i++)
    files.Add(new TFileDroplet(drop, i));

  AddFiles(files);

  drop.DragFinish();
}


void TBUTTONApp::AddFiles(TFileDropletList& files)
{
  for (TFileDropletListIter fileIter(files); fileIter; fileIter++) {
    TDocTemplate* tpl = GetDocManager()->MatchTemplate(fileIter.Current()->GetName());
    if (tpl)
      GetDocManager()->CreateDoc(tpl, fileIter.Current()->GetName());
  }
}


void TBUTTONApp::EvOwlDocument(TDocument& doc)
{
  if (doc.GetDocPath())
    SaveMenuChoice(doc.GetDocPath());
}

int32 TBUTTONApp::CmFileSelected(uint wp, int32)
{
  TAPointer<char> text = new char[_MAX_PATH];

  GetMenuText(wp, text, _MAX_PATH);
  TDocTemplate* tpl = GetDocManager()->MatchTemplate(text);
  if (tpl)
    GetDocManager()->CreateDoc(tpl, text);
  return 0;
}

bool TBUTTONApp::ProcessAppMsg(MSG& msg)
{
  if (msg.message == WM_COMMAND) {
    if (ContextHelp || ::GetKeyState(VK_F1) < 0) {
      ContextHelp = false;
      GetMainWindow()->WinHelp(HelpFileName, HELP_CONTEXT, msg.wParam);
      return true;
    }
  }
  else
    switch(msg.message) {
    case WM_KEYDOWN:
      if (msg.wParam == VK_F1) {
        if (::GetKeyState(VK_SHIFT) < 0) {
          ContextHelp = true;
          HelpCursor = new TCursor(GetMainWindow()->GetModule()->GetInstance(), TResId(IDC_HELPCURSOR));
          ::SetCursor(*HelpCursor);
          return true;
        }
        else {
          GetMainWindow()->WinHelp(HelpFileName, HELP_INDEX, 0);
          return true;
        }
      }
      else {
        if (ContextHelp && msg.wParam == VK_ESCAPE) {
          if (HelpCursor)
            delete HelpCursor;
          HelpCursor = 0;
          ContextHelp = false;
          GetMainWindow()->SetCursor(0, IDC_ARROW);
          return true;
        }
      }
      break;

    case WM_MOUSEMOVE:
    case WM_NCMOUSEMOVE:
      if (ContextHelp) {
        ::SetCursor(*HelpCursor);
        return true;
      }
      break;

    case WM_INITMENU:
      if (ContextHelp) {
        ::SetCursor(*HelpCursor);
        return true;
      }
      break;
    case WM_ENTERIDLE:
      if (msg.wParam == MSGF_MENU)
        if (GetKeyState(VK_F1) < 0) {
          ContextHelp = true;
          GetMainWindow()->PostMessage(WM_KEYDOWN, VK_RETURN, 0);
          return true;
        }
      break;
    default:
      ;
    }

  return TApplication::ProcessAppMsg(msg);
}


void TBUTTONApp::EvWinIniChange(char far* section)
{
  if (strcmp(section, "windows") == 0) {
    if (Printer) {
      const int bufferSize = 255;
      char printDBuffer[bufferSize];
      LPSTR printDevice = printDBuffer;
      LPSTR devName;
      LPSTR driverName = 0;
      LPSTR outputName = 0;
      if (::GetProfileString("windows", "device", "", printDevice, bufferSize)) {
        devName = printDevice;
        while (*printDevice) {
          if (*printDevice == ',') {
            *printDevice++ = 0;
            if (!driverName)
              driverName = printDevice;
            else
              outputName = printDevice;
          }
          else
            printDevice = ::AnsiNext(printDevice);
        }

        if (Printer->GetSetup().Error != 0                ||
            strcmp(devName, Printer->GetSetup().GetDeviceName()) != 0  ||
            strcmp(driverName, Printer->GetSetup().GetDriverName()) != 0 ||
            strcmp(outputName, Printer->GetSetup().GetOutputName()) != 0 ) {
          delete Printer;
          Printer = new TPrinter(this);
        }
      }
      else {
        delete Printer;
        Printer = new TPrinter(this);
      }
    }
  }
}


int OwlMain(int , char* [])
{
  TBUTTONApp   app;
  return app.Run();
}


/********************************************************************************
***                                                                           ***
***           NEW CODE FOR TOOLBARS SUPPORT                                   ***
***                                                                           ***
***   (DON'T DELETE THESE LINES : THEY ARE CALLED FROM CUSTOMIZE DIALOG).     ***
***                                                                           ***
********************************************************************************/

bool TBUTTONApp::InsertGadgetInToolbar(int gadgid, TGadgetWindow* wind, TGadgetWindow::TPlacement place, TGadget* gadgpos)
{
  bool result = false;

  if (wind)
     {
     if (gadgid != 0)
        {
        TGadget* gadg = GadgetAvailableList->GadgetWithId(gadgid);
        if (gadg)
            {
            TGadget* gadgmove = GadgetAvailableList->Remove(*gadg);
            if (gadgmove)
               {
               TControlGadget* ctlgadg = TYPESAFE_DOWNCAST(gadgmove, TControlGadget);
               if (ctlgadg)    //  If the gadget is a control, show this control
                  {
                  TWindow* window = ctlgadg->GetControl();
                  if ((window) && (wind->IsWindowVisible()))
                     window->ShowWindow(SW_SHOW);
                  }
               if (gadgpos == 0)
                  wind->Insert(*gadgmove);
               else wind->Insert(*gadgmove, place, gadgpos);
               result = true;
               }
            }
        }
     else
        {
        result = true;
        if (gadgpos)
           wind->Insert(*new JPFlatDivGadget(0, divGadget, divEOL, divVertBar), place, gadgpos);
        else
           wind->Insert(*new JPFlatDivGadget(0, divGadget, divEOL, divVertBar));
        }
     }

  return result;
}

bool TBUTTONApp::RemoveGadgetFromToolbar(TGadget* gadg, TGadgetWindow* wind, TGadgetWindow::TPlacement place, TGadget* gadgpos)
{
  bool result = false;

  if ((gadg) && (wind))
     {
     int id = gadg->GetId();
     TGadget* gadgmove = wind->Remove(*gadg);
     if (gadgmove)
         {
         TControlGadget* ctlgadg = TYPESAFE_DOWNCAST(gadgmove, TControlGadget);
         if (ctlgadg)    //  If the gadget is a control, hides this control
            {
            TWindow* window = ctlgadg->GetControl();
            if (window)
               window->ShowWindow(SW_HIDE);
            }
         if (id != 0)
            {
            if (gadgpos == 0)
               GadgetAvailableList->Insert(*gadgmove);
            else GadgetAvailableList->Insert(*gadgmove, place, gadgpos);
            }
         else
            delete gadgmove;
         result = true;
         }
     }

  return result;
}

/***   Insert the default separator at the top of the available list    ***/
void TBUTTONApp::InsertSeparatorInAvailableList()
{
  TGadget* firstgadg = GadgetAvailableList->FirstGadget();
  InsertGadgetInToolbar(0, GadgetAvailableList, TGadgetWindow::Before, firstgadg);
}

bool TBUTTONApp::RemoveSeparatorFromAvailableList()
{
  bool result = false;
  bool canCont = true;

  while (canCont)
       {
       TGadget* gadg = GadgetAvailableList->GadgetWithId(0);
       if (gadg)
          canCont = RemoveGadgetFromToolbar(gadg, GadgetAvailableList);
       else canCont = false;
       }

  return result;
}

void TBUTTONApp::StoreToolbarDataInProfile()
{
   uint count;
   char sectbuf[255];
   char keybuf[255];
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      wsprintf(sectbuf, "Toolbar%u", i);
      TProfile toolbarsection (sectbuf, IniFile);
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         TGadgetWindow::TTileDirection windir = wind->GetDirection();
         TRect wrect = wind->GetWindowRect();
         TRect frect = frame->GetWindowRect();
         if ((windir == TGadgetWindow::Horizontal) && (wrect.right > frect.right))
            wrect.right = frect.right;
         bool contain = wrect.Contains(frect);
         TPoint point = TPoint(wrect.left, wrect.top);
         frame->ScreenToClient(point);
         count = 0;
         toolbarsection.WriteString(NULL, NULL); /*     deletes the entire section   */
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           int id = g->GetId();
           wsprintf(keybuf, "Button%u", count);
           toolbarsection.WriteInt(keybuf, id);
           count++;
           }
         }
      }
}

void TBUTTONApp::LoadToolbarDataFromProfile(bool updatewindow)
{
   uint count;
   char sectbuf[255];
   char keybuf[255];
   bool onekeyfound = false;
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      wsprintf(sectbuf, "Toolbar%u", i);
      TProfile toolbarsection (sectbuf, IniFile);
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         count = 0;
         bool endofsection = false;
         while (endofsection == false)
           {
           wsprintf(keybuf, "Button%u", count);
           int gadgid = toolbarsection.GetInt(keybuf, -1);
           if (gadgid != -1)
              {
              if ((InsertGadgetInToolbar(gadgid, wind)) && (onekeyfound == false))
                 onekeyfound = true;
              }
           else endofsection = true;
           count++;
           }
         if ((count > 0) && (updatewindow))
            {
            wind->Invalidate();
            wind->LayoutSession();
            }
         }
      }
   if (onekeyfound == true)
      InsertSeparatorInAvailableList();
   else LoadToolbarDataFromArray();
}

void TBUTTONApp::LoadToolbarDataFromArray(bool updatewindow)
{
  int maxnum = sizeof(GadgetList) / sizeof(TOOLBARDATA);
  for (int i = 0; i < maxnum; i++)
      {
      TOOLBARDATA data = GadgetList[i];
      int gadgid = data.GadgetId;
      int toolid = data.ToolbarPos;
      TGadgetWindow* gadgwin = ToolbarList[toolid];
      if (gadgwin)
         InsertGadgetInToolbar(gadgid, gadgwin);
      }
  if (updatewindow)
      {
      for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
         {
         TGadgetWindow* wind = ToolbarList[i];
         if (wind)
            {
            wind->Invalidate();
            wind->LayoutSession();
            }
         }
      }
  InsertSeparatorInAvailableList();
}

void TBUTTONApp::ReloadToolbarData()
{
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      bool canCont = true;
      if (wind)
         {
         TGadget* g;
         g = wind->FirstGadget();
         while ((g) && (canCont))
               {
               canCont = RemoveGadgetFromToolbar(g, wind);
               g = wind->FirstGadget();
               }
         }
      }
   RemoveSeparatorFromAvailableList();
   LoadToolbarDataFromArray(true);
}

TGadget* TBUTTONApp::FindGadgetInToolbars(int id)
{
   TGadget* result = NULL;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if ((wind) && (!result))
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           if (g->GetId() == id)
              {
              result = g;
              break;
              }
           }
         }
      }
   return result;
}

/********************************************************************************
***                                                                           ***
***           NEW CODE FOR TOOLBARS SUPPORT                                   ***
***                                                                           ***
***   (YOU CAN DELETE THESE LINES IF YOU DON'T NEED A FULL SUPPORT).          ***
***                                                                           ***
********************************************************************************/
void TBUTTONApp::SetDisplayType(JPFlatButtonTextGadget::JPDisplayType disp)
{
   dispType = disp;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetDisplayType(disp);
           }
         }
      }
}

void TBUTTONApp::SetTextPosition(JPFlatButtonTextGadget::JPTextPos pos)
{
   textPos = pos;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetTextPosition(pos);
           }
         }
      }
}

void TBUTTONApp::SetTextRows(uint rows)
{
   ToolbarRows = rows;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetTextRows(rows);
           }
         }
      }
}

void TBUTTONApp::SetMaxChars(uint maxchars)
{
   MaxChars = maxchars;
   if (txtGadg)
      {
      char buf[255];
      wsprintf(buf, "Chars = %u", MaxChars);
      txtGadg->SetText(buf);
      }
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetMaxChars(maxchars);
           }
         }
      }
}

void TBUTTONApp::SetDivId(TGadget* id)
{
   divGadget = id;
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatDivGadget* divg = TYPESAFE_DOWNCAST(g, JPFlatDivGadget);
           if (divg)
              divg->SetReferenceGadget(divGadget);
           }
         }
      }
}

void TBUTTONApp::CeToolbarbitmap(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((dispType == JPFlatButtonTextGadget::Bitmap) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbarbitmaptext(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((dispType == JPFlatButtonTextGadget::BitmapText) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbarbottom(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((textPos == JPFlatButtonTextGadget::Bottom) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbarleft(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((textPos == JPFlatButtonTextGadget::Left) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbarright(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((textPos == JPFlatButtonTextGadget::Right) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbartext(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((dispType == JPFlatButtonTextGadget::Text) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CeToolbartop(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((textPos == JPFlatButtonTextGadget::Top) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CeOneTextRow(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((ToolbarRows == 1) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CeTwoTextRow(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((ToolbarRows == 2) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmToolbarbitmap()
{
  // INSERT>> Your code here.
  SetDisplayType(JPFlatButtonTextGadget::Bitmap);
}


void TBUTTONApp::CmToolbarbitmaptext()
{
  // INSERT>> Your code here.
  SetDisplayType(JPFlatButtonTextGadget::BitmapText);
}


void TBUTTONApp::CmToolbarbottom()
{
  // INSERT>> Your code here.
  SetTextPosition(JPFlatButtonTextGadget::Bottom);
}


void TBUTTONApp::CmToolbarleft()
{
  // INSERT>> Your code here.
  SetTextPosition(JPFlatButtonTextGadget::Left);
}


void TBUTTONApp::CmToolbarright()
{
  // INSERT>> Your code here.
  SetTextPosition(JPFlatButtonTextGadget::Right);
}


void TBUTTONApp::CmToolbartext()
{
  // INSERT>> Your code here.
  SetDisplayType(JPFlatButtonTextGadget::Text);
}


void TBUTTONApp::CmToolbartop()
{
  // INSERT>> Your code here.
  SetTextPosition(JPFlatButtonTextGadget::Top);
}

void TBUTTONApp::CmOneTextRow()
{
  // INSERT>> Your code here.
  SetTextRows(1);
}

void TBUTTONApp::CmTwoTextRow()
{
  // INSERT>> Your code here.
  SetTextRows(2);
}


void TBUTTONApp::CmDecreasechars()
{
  // INSERT>> Your code here.
  if (MaxChars > 0)
     SetMaxChars(--MaxChars);
}


void TBUTTONApp::CeDecreasechars(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if ((dispType != JPFlatButtonTextGadget::Bitmap) && (MaxChars > 0))
     tce.Enable(true);
  else tce.Enable(false);
}


void TBUTTONApp::CmDefaultchars()
{
  // INSERT>> Your code here.
  SetMaxChars(4);
}


void TBUTTONApp::CeDefaultchars(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck((MaxChars == 4) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CmIncreasechars()
{
  // INSERT>> Your code here.
  if (MaxChars <= 10)
     SetMaxChars(++MaxChars);
}


void TBUTTONApp::CeIncreasechars(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if ((dispType != JPFlatButtonTextGadget::Bitmap) && (MaxChars <= 10))
     tce.Enable(true);
  else tce.Enable(false);
}


void TBUTTONApp::CmHottracking()
{
  // INSERT>> Your code here.
   HotTrack ^= 1;
   uint maxnum = ToolbarList.GetItemsInContainer();
   TGadgetWindow* wind;
   for (uint i = 0; i <= maxnum; i++)
      {
      if (i < maxnum)
         wind = ToolbarList[i];
      else wind = GadgetAvailableList;
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatButtonTextGadget* bg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
           if (bg)
              bg->SetHotTrack(HotTrack);
           }
         }
      }
}

void TBUTTONApp::CeHottracking(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  if (dispType != JPFlatButtonTextGadget::Bitmap)
     tce.Enable(true);
  else tce.Enable(false);
  tce.SetCheck(HotTrack ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmDivbuttons()
{
  // INSERT>> Your code here.
  SetDivId(FindGadgetInToolbars(CM_MDIFILENEW));
}

void TBUTTONApp::CeDivbuttons(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((divGadget == FindGadgetInToolbars(CM_MDIFILENEW)) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmDivcombo()
{
  // INSERT>> Your code here.
  SetDivId(FindGadgetInToolbars(ID_COMBOBOX));
}

void TBUTTONApp::CeDivcombo(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((divGadget == FindGadgetInToolbars(ID_COMBOBOX)) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmDivnothing()
{
  // INSERT>> Your code here.
  SetDivId(0);
}

void TBUTTONApp::CeDivnothing(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck((divGadget == 0) ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}

void TBUTTONApp::CmDivendofline()
{
  // INSERT>> Your code here.
   divEOL ^= 1;
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatDivGadget* divg = TYPESAFE_DOWNCAST(g, JPFlatDivGadget);
           if (divg)
              divg->SetEndOfLineStatus(divEOL);
           }
         }
      }
}

void TBUTTONApp::CeDivendofline(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck(divEOL ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CmDivvertbar()
{
  // INSERT>> Your code here.
   divVertBar ^= 1;
   for (uint i = 0; i < ToolbarList.GetItemsInContainer(); i++)
      {
      TGadgetWindow* wind = ToolbarList[i];
      if (wind)
         {
         for (TGadget* g = wind->FirstGadget(); g; g = wind->NextGadget(*g))
           {
           JPFlatDivGadget* divg = TYPESAFE_DOWNCAST(g, JPFlatDivGadget);
           if (divg)
              divg->SetNoVertBarStatus(divVertBar);
           }
         }
      }
}


void TBUTTONApp::CeDivvertbar(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
  tce.SetCheck(divVertBar ? TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void TBUTTONApp::CmToolbarcustomize()
{
  // INSERT>> Your code here.
  JPCustomizeToolbar(GetMainWindow()).Execute();
}


void TBUTTONApp::CeToolbarcustomize(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
}


void TBUTTONApp::CeButtonMenu(TCommandEnabler &tce)
{
  // INSERT>> Your code here.
  tce.Enable(true);
}

