/*******************************************************************************
*   Project BUTTON                                                             *
*   http://space.tin.it/computer/giparrel, joparrel@tin.it                     *
*   Copyright  1998. All Rights Reserved.                                     *
*                                                                              *
*   SUBSYSTEM:    BUTTON Application                                           *
*   FILE:         buttonap.h                                                   *
*   AUTHOR:       Jo Parrello.                                                 *
*                                                                              *
*   OVERVIEW                                                                   *
*   ~~~~~~~~----                                                               *
*   Class definition for TBUTTONApp (TApplication).                            *
*                                                                              *
*   Revision:   1.0  (May 15, 1998).                                           *
*                                                                              *
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF          *
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO        *
*   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A             *
*   PARTICULAR PURPOSE.                                                        *
*******************************************************************************/
#if !defined(buttonap_h)              // Sentry, use file only if it's not already included.
#define buttonap_h

#include <owl/controlb.h>
#include <owl/docking.h>
#include <owl/printer.h>
#include <owl/rcntfile.h>
#include <owl/textgadg.h>
#include <owl/statusba.h>

#include <classlib/bags.h>
#include <classlib/arrays.h>

#include "jpflbute.h"
#include "jpfltdiv.h"
#include "jpfltbox.h"
#include "buttonmd.h"

#include "buttonap.rh"            // Definition of all resources.

//{{TApplication = TBUTTONApp}}
class TBUTTONApp : public TApplication, public TRecentFiles {
  private:
    bool            HelpState;                          // Has the help engine been used.
    bool            ContextHelp;                        // SHIFT-F1 state(context sensitive HELP).
    TCursor*        HelpCursor;                         // Context sensitive help cursor.

    void SetupSpeedBar(TDecoratedMDIFrame* frame, TGadgetWindow::TTileDirection direction, TFont* font = NULL);
    void AddFiles(TFileDropletList& files);
    void ProcessCmdLine(char * CmdLine);
    void RegisterInfo();
    void UnRegisterInfo();

  public:
    TBUTTONApp();
    virtual ~TBUTTONApp();

    void CreateGadgets(TDockableControlBar* cb, TFont* font = NULL, bool server = false);
    TDockableControlBar* cb;
    TDockableControlBar* custombar;
    TIArrayAsVector<TGadgetWindow> ToolbarList;
    TGadgetWindow*  GadgetAvailableList;
    TDecoratedMDIFrame* frame;
    THarbor*        ApxHarbor;
    TTextGadget*    txtGadg;
    TFont*          toolbarFont;
    JPFlatButtonTextGadget::JPDisplayType   dispType, DefaultDispType;
    JPFlatButtonTextGadget::JPTextPos       textPos, DefaultTextPos;
    uint            ToolbarRows, MaxChars;
    bool            HotTrack;
    JPFlatComboBox* flatcombo;
    TGadget*        divGadget;
    bool            divEOL;
    bool            divVertBar;

    TBUTTONMDIClient* MdiClient;

    // Public data members used by the print menu commands and Paint routine in MDIChild.
    //
    TPrinter*       Printer;                           // Printer support.
    int             Printing;                          // Printing in progress.

//{{TBUTTONAppVIRTUAL_BEGIN}}
  public:
    virtual void InitMainWindow();
    virtual void InitInstance();
    virtual bool CanClose();
    virtual bool ProcessAppMsg(MSG& msg);
    /****    ADDED TO SUPPORT MULTIPLE TOOLBARS     */
    virtual bool InsertGadgetInToolbar(int gadgid, TGadgetWindow* wind, TGadgetWindow::TPlacement place = TGadgetWindow::After, TGadget* gadgpos = 0);
    virtual bool RemoveGadgetFromToolbar(TGadget* gadg, TGadgetWindow* wind, TGadgetWindow::TPlacement place = TGadgetWindow::After, TGadget* gadgpos = 0);
    virtual void InsertSeparatorInAvailableList();
    virtual bool RemoveSeparatorFromAvailableList();
    virtual void LoadToolbarDataFromProfile(bool updatewindow = false);
    virtual void LoadToolbarDataFromArray(bool updatewindow = false);
    virtual void ReloadToolbarData();
    virtual TGadget* FindGadgetInToolbars(int id);
    virtual void StoreToolbarDataInProfile();
    virtual void SetDisplayType(JPFlatButtonTextGadget::JPDisplayType disp);
    virtual void SetTextPosition(JPFlatButtonTextGadget::JPTextPos pos);
    virtual void SetTextRows(uint rows);
    virtual void SetMaxChars(uint maxchars);
    virtual void SetDivId(TGadget* id);
//{{TBUTTONAppVIRTUAL_END}}

//{{TBUTTONAppRSP_TBL_BEGIN}}
  protected:
    void EvNewView(TView& view);
    void EvCloseView(TView& view);
    void CmHelpAbout();
    void CmHelpContents();
    void CmHelpUsing();
    void EvDropFiles(TDropInfo drop);
    void EvWinIniChange(char far* section);
    void EvOwlDocument(TDocument& doc);
    int32 CmFileSelected(uint wp, int32 lp);
    void CeToolbarbitmap(TCommandEnabler &tce);
    void CeToolbarbitmaptext(TCommandEnabler &tce);
    void CeToolbarbottom(TCommandEnabler &tce);
    void CeToolbarleft(TCommandEnabler &tce);
    void CeToolbarright(TCommandEnabler &tce);
    void CeToolbartext(TCommandEnabler &tce);
    void CeToolbartop(TCommandEnabler &tce);
    void CeOneTextRow(TCommandEnabler &tce);
    void CeTwoTextRow(TCommandEnabler &tce);
    void CmToolbarbitmap();
    void CmToolbarbitmaptext();
    void CmToolbarbottom();
    void CmToolbarleft();
    void CmToolbarright();
    void CmToolbartext();
    void CmToolbartop();
    void CmOneTextRow();
    void CmTwoTextRow();
    void CmDecreasechars();
    void CeDecreasechars(TCommandEnabler &tce);
    void CmDefaultchars();
    void CeDefaultchars(TCommandEnabler &tce);
    void CmIncreasechars();
    void CeIncreasechars(TCommandEnabler &tce);
    void CmHottracking();
    void CeHottracking(TCommandEnabler &tce);
    void CmDivbuttons();
    void CeDivbuttons(TCommandEnabler &tce);
    void CmDivcombo();
    void CeDivcombo(TCommandEnabler &tce);
    void CmDivnothing();
    void CeDivnothing(TCommandEnabler &tce);
    void CmDivendofline();
    void CeDivendofline(TCommandEnabler &tce);
    void CmDivvertbar();
    void CeDivvertbar(TCommandEnabler &tce);
    void CmToolbarcustomize();
    void CeToolbarcustomize(TCommandEnabler &tce);
    void CeButtonMenu(TCommandEnabler &tce);
//{{TBUTTONAppRSP_TBL_END}}
DECLARE_RESPONSE_TABLE(TBUTTONApp);
};    //{{TBUTTONApp}}


#endif  // buttonap_h sentry.
