//----------------------------------------------------------------------------
//  Project BUTTON
//  http://space.tin.it/computer/giparrel
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    BUTTON Application
//  FILE:         buttonmd.cpp
//  AUTHOR:       Jo Parrello.
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TBUTTONMDIClient (TMDIClient).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <owl/docmanag.h>
#include <owl/listbox.h>
#include <stdio.h>

#include "buttonap.h"
#include "buttonm1.h"
#include "buttonmd.h"
#include "apxprint.h"
#include "apxprev.h"


//{{TBUTTONMDIClient Implementation}}


//
// Build a response table for all messages/commands handled
// by TBUTTONMDIClient derived from TMDIClient.
//
DEFINE_RESPONSE_TABLE1(TBUTTONMDIClient, TMDIClient)
//{{TBUTTONMDIClientRSP_TBL_BEGIN}}
  EV_COMMAND(CM_FILEPRINT, CmFilePrint),
  EV_COMMAND(CM_FILEPRINTERSETUP, CmFilePrintSetup),
  EV_COMMAND(CM_FILEPRINTPREVIEW, CmFilePrintPreview),
  EV_COMMAND_ENABLE(CM_FILEPRINT, CmPrintEnable),
  EV_COMMAND_ENABLE(CM_FILEPRINTERSETUP, CmPrintEnable),
  EV_COMMAND_ENABLE(CM_FILEPRINTPREVIEW, CmPrintEnable),
  EV_WM_DROPFILES,
//{{TBUTTONMDIClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TBUTTONMDIClient
// ~~~~~~~~~~~
// Construction/Destruction handling.
//
TBUTTONMDIClient::TBUTTONMDIClient(TModule* module)
:
  TMDIClient(module)
{
  ChildCount = 0;

  // INSERT>> Your constructor code here.

}


TBUTTONMDIClient::~TBUTTONMDIClient()
{
  Destroy();

  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TBUTTONMDIClient
// ~~~~~~~~~~~
// MDIClient site initialization.
//
void TBUTTONMDIClient::SetupWindow()
{
  // Default SetUpWindow processing.
  //
  TMDIClient::SetupWindow();

  // Accept files via drag/drop in the client window.
  //
  DragAcceptFiles(true);
}


//--------------------------------------------------------
// TBUTTONMDIClient
// ~~~~~~~~~~
// Menu File Print command
//
void TBUTTONMDIClient::CmFilePrint()
{
  // Create Printer object if not already created.
  //
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp) {
    if (!theApp->Printer)
      theApp->Printer = new TPrinter(theApp);

    TAPointer<char> docName = new char[_MAX_PATH];

    TDocument* currentDoc = theApp->GetDocManager()->GetCurrentDoc();
    if (currentDoc->GetTitle())
      strcpy(docName, currentDoc->GetTitle());
    else
      strcpy(docName, "Document");

    // Create Printout window and set characteristics.
    //
    TApxPrintout printout(theApp->Printer, docName, GetActiveMDIChild()->GetClientWindow(), true);

    theApp->Printing++;

    // Bring up the Print dialog and print the document.
    //
    theApp->Printer->Print(GetWindowPtr(GetActiveWindow()), printout, true);

    theApp->Printing--;
  }
}


//--------------------------------------------------------
// TBUTTONMDIClient
// ~~~~~~~~~~
// Menu File Print Setup command
//
void TBUTTONMDIClient::CmFilePrintSetup()
{
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp) {
    if (!theApp->Printer)
      theApp->Printer = new TPrinter(theApp);

    // Bring up the Print Setup dialog.
    //
    theApp->Printer->Setup(this);
  }
}


//--------------------------------------------------------
// TBUTTONMDIClient
// ~~~~~~~~~~
// Menu File Print Preview command
//
void TBUTTONMDIClient::CmFilePrintPreview()
{
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp) {
    if (!theApp->Printer)
      theApp->Printer = new TPrinter(GetApplication());

    theApp->Printing++;

    TApxPreviewWin* prevW = new TApxPreviewWin(Parent, theApp->Printer, GetActiveMDIChild()->GetClientWindow(), "Print Preview", new TLayoutWindow(0));
    prevW->Create();

    // Here we resize the preview window to take the size of the MainWindow, then
    // hide the MainWindow.
    //
    TFrameWindow * mainWindow =  GetApplication()->GetMainWindow();
    prevW->SetExStyle(prevW->GetExStyle() | WS_EX_APPWINDOW);
    WINDOWPLACEMENT mainPos;
    mainPos.length = sizeof(WINDOWPLACEMENT);  // to solve a horrific bug on TWindow class
    mainWindow->GetWindowPlacement(&mainPos);
    prevW->SetWindowPlacement(&mainPos);
    mainWindow->ShowWindow(SW_HIDE);
    prevW->Invalidate();
    //prevW->UpdateWindow();

    GetApplication()->BeginModal(GetApplication()->GetMainWindow());

    // After the user closes the preview Window, we take its size and use it
    // to size the MainWindow, then show the MainWindow again.
    //
    mainPos.length = sizeof(WINDOWPLACEMENT);  // to solve a horrific bug on TWindow class
    prevW->GetWindowPlacement(&mainPos);
    mainWindow->SetWindowPlacement(&mainPos);

    // We must destroy the preview window explicitly.  Otherwise, the window will
    // not be destroyed until it's parent the MainWindow is destroyed.
    //
    prevW->Destroy();
    delete prevW;

    theApp->Printing--;
  }
}


//--------------------------------------------------------
// TBUTTONMDIClient
// ~~~~~~~~~~
// Menu enabler used by Print, Print Setup and Print Preview.
//
void TBUTTONMDIClient::CmPrintEnable(TCommandEnabler& tce)
{
  if (GetActiveMDIChild()) {
    TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
    if (theApp) {
      // If we have a Printer already created just test if all is okay.
      // Otherwise create a Printer object and make sure the printer really
      // exists and then delete the Printer object.
      //
      if (!theApp->Printer) {
        theApp->Printer = new TPrinter(theApp);
        tce.Enable(!theApp->Printer->GetSetup().Error);
      }
      else
        tce.Enable(!theApp->Printer->GetSetup().Error);
    }
  }
  else
    tce.Enable(false);
}


void TBUTTONMDIClient::EvDropFiles(TDropInfo)
{
  Parent->ForwardMessage();
}
