/*******************************************************************************
*   Project BUTTON                                                             *
*   http://space.tin.it/computer/giparrel, joparrel@tin.it                     *
*   Copyright  1998. All Rights Reserved.                                     *
*                                                                              *
*   SUBSYSTEM:    button.apx Application                                       *
*   FILE:         jpbtnlst.cpp                                                 *
*   AUTHOR:       Jo Parrello.                                                 *
*                                                                              *
*   OVERVIEW                                                                   *
*   ~~~~~~~~----                                                               *
*   Source file for implementation of JPButtonList (TListBox).                 *
*                                                                              *
*   Revision:   1.0  (May 15, 1998).                                           *
*                                                                              *
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF          *
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO        *
*   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A             *
*   PARTICULAR PURPOSE.                                                        *
*******************************************************************************/
#include <owl/pch.h>
#include <owl/celarray.h>
#include <owl/gadgetwi.h>
#include <owl/controlg.h>
#include <owl/textgadg.h>

#include "jpflbute.h"
#include "jpbtnlst.h"

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(JPButtonList, TListBox)
//{{JPButtonListRSP_TBL_BEGIN}}
  EV_WM_MEASUREITEM,
//{{JPButtonListRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{JPButtonList Implementation}}


JPButtonList::JPButtonList(TWindow* parent, int resourceId, TGadgetWindow* gadgWin, TModule* module)
:
    TListBox(parent, resourceId, module)
{
  // INSERT>> Your constructor code here.
  Attr.Style &= ~(LBS_SORT | LBS_HASSTRINGS | LBS_MULTIPLESEL);
  Attr.Style |= (LBS_NOTIFY | WS_VSCROLL | WS_HSCROLL | WS_BORDER | LBS_OWNERDRAWVARIABLE);
  sizex = sizey = 0;
  gadgetWindow = gadgWin;
  SetBkgndColor(TColor::Sys3dFace);
}


JPButtonList::~JPButtonList()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

}

void JPButtonList::MeasureItem (MEASUREITEMSTRUCT far& measureInfo)
{
  if (sizey == 0)
     {
     TSize tempsize;
     GetDesiredSize(tempsize);
     if (tempsize.cx == 0)
        sizex = 22;
     else sizex = tempsize.cx;
     if (tempsize.cy == 0)
        sizey = 22;
     else sizey = tempsize.cy;
     }
  measureInfo.itemHeight = sizey;
}

void JPButtonList::SetupWindow()
{
  TListBox::SetupWindow();

  // INSERT>> Your code here.
  StoreDataInListBox();
}

void JPButtonList::StoreDataInListBox()
{
  if (gadgetWindow)
     {
     for (TGadget* g = gadgetWindow->FirstGadget(); g; g = gadgetWindow->NextGadget(*g))
        {
        int id = g->GetId();
        AddString((LPCTSTR) id);
        }
     if (GetCount() > 0)
        SetSelIndex(0);
     }
}

void JPButtonList::DrawItem(DRAWITEMSTRUCT far& drawInfo)
{
  if (gadgetWindow)
     {
     char buf[250] = "";
     string gadgtext;
     TDC dc(drawInfo.hDC);
     if (drawInfo.itemData == 0)
        {                         /*      THIS GADGET IS A DIVIDER GADGET        */
        TRect rect = drawInfo.rcItem;
        rect.right = sizex;
        rect.Inflate(-1,-1);
        int cy = rect.top + ((rect.bottom - rect.top) / 2);
        TPen pen (TColor::Sys3dShadow);
        dc.SelectObject(pen);
        dc.MoveTo(rect.left, cy);
        dc.LineTo(rect.right, cy);
        dc.RestorePen();
        cy++;
        TPen pen2 (TColor::Sys3dHilight);
        dc.SelectObject(pen2);
        dc.MoveTo(rect.left, cy);
        dc.LineTo(rect.right, cy);
        dc.RestorePen();
        wsprintf(buf, "Separator");
        }
     else
        {
        TGadget* gadg = gadgetWindow->GadgetWithId((int) drawInfo.itemData);
        if (gadg)
           {
           JPFlatButtonTextGadget* butg = TYPESAFE_DOWNCAST(gadg, JPFlatButtonTextGadget);
           if (butg)     /*  THIS GADGET IS A FLAT BUTTON GADGET   */
              {
              TDib* Dib = butg->GetDefaultGlyphDib();
              TRect imageRect = TRect(0, 0, Dib->Width(), Dib->Height());
              TRect rect = drawInfo.rcItem;
              rect.right = sizex;
              rect.Inflate(-1,-1);
              rect.Offset(1,0);
              dc.StretchDIBits(rect, imageRect, *Dib, SRCCOPY);
              TBrush brush (TColor::Sys3dHilight);
              dc.FrameRect(rect, brush);
              TPen pen (TColor::Sys3dShadow);
              dc.SelectObject(pen);
              dc.MoveTo(rect.right - 1, rect.top);
              dc.LineTo(rect.right - 1, rect.bottom - 1);
              dc.LineTo(rect.left,  rect.bottom - 1);
              dc.RestorePen();
              delete Dib;
              gadgtext = LoadInternalString((uint) drawInfo.itemData);
              if (gadgtext.length() > 0)
                 wsprintf(buf, "%s", gadgtext.c_str());
              else
                 wsprintf(buf, "Button Gadget Id = %lu", drawInfo.itemData);
              }
           TControlGadget* ctlg = TYPESAFE_DOWNCAST(gadg, TControlGadget);
           if (ctlg)     /*  THIS GADGET IS A CONTROL GADGET   */
              {
              char tempbuf[250];
              TWindow* control = ctlg->GetControl();
              int templen = 0;
              if (control)
                  {
                  control->GetClassName(tempbuf, sizeof(tempbuf));
                  templen = strlen(tempbuf);
                  }
              gadgtext = LoadInternalString((uint) drawInfo.itemData);
              if ((templen > 0) && (gadgtext.length() > 0))
                 wsprintf(buf, "%s (%s)", gadgtext.c_str(), tempbuf);
              else
                 {
                 if ((templen == 0) && (gadgtext.length() > 0))
                    wsprintf(buf, "%s (Control)", gadgtext.c_str());
                 else
                    {
                    if ((templen > 0) && (gadgtext.length() == 0))
                       wsprintf(buf, "Control (%s)", tempbuf);
                    else
                       wsprintf(buf, "Control Id = %lu", drawInfo.itemData);
                    }
                 }
              }
           TTextGadget* txtg = TYPESAFE_DOWNCAST(gadg, TTextGadget);
           if (txtg)     /*  THIS GADGET IS A TEXT GADGET   */
              {
              gadgtext = LoadInternalString((uint) drawInfo.itemData);
              if (gadgtext.length() > 0)
                 wsprintf(buf, "%s (Text)", gadgtext.c_str());
              else
                 {
                 gadgtext = (string) txtg->GetText();
                 if (gadgtext.length() > 0)
                    wsprintf(buf, "%s (Text)", gadgtext.c_str());
                 else
                    wsprintf(buf, "Text Gadget Id = %lu", drawInfo.itemData);
                 }
              }
           if (strlen(buf) == 0)
               wsprintf(buf, "Gadget Id = %lu", drawInfo.itemData);
           }
        }
     TColor back = !(drawInfo.itemState & ODS_SELECTED) ? TColor::Sys3dFace : TColor::SysHighlight;
     TColor text = !(drawInfo.itemState & ODS_SELECTED) ? TColor::SysWindowText : TColor::SysHighlight;
     TBrush br(back);
     TRect rect = drawInfo.rcItem;
     rect.Inflate(-1, 0);
     dc.FrameRect(rect, br);
     back = TColor::Sys3dFace;
     if (drawInfo.itemAction & ODA_FOCUS)
        {
        TRect frect = rect;
        frect.left = sizex;
        frect.right = frect.right - 2;
        frect.Inflate(0, -1);
        if (drawInfo.itemState & ODS_FOCUS)
           {
           dc.FillRect(frect, br);
           back = TColor::SysHighlight;
           text = TColor::SysHighlightText;
           }
        else dc.FillRect(frect, TBrush(TColor::Sys3dFace));
        }
     dc.SetBkColor(back);
     dc.SetTextColor(text);
     rect.left = sizex + 5;
     dc.DrawText(buf, strlen(buf), rect, DT_VCENTER | DT_LEFT | DT_SINGLELINE);
     dc.RestoreObjects();
     }
}

string JPButtonList::LoadInternalString(uint Id)
{
   char tempstr[256], szBuf[256];
   char* ptr = NULL, c = '\n';
   strcpy(szBuf,"");
   strcpy(tempstr,"");
   int len = GetModule()->LoadString(Id, tempstr, sizeof(szBuf));
   if (len > 0)
      ptr = strchr(tempstr, c);
   if (ptr)
      strncpy(szBuf, ptr + 1, min(len, (int) sizeof(szBuf)));
   else
      {
      TWindow* parent= GetParentO();
      TDecoratedFrame* frame= parent ? TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
      while (parent && !frame)
        {
        parent = parent->GetParentO();
        if (parent)
          frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
        }
      if (frame)
          frame->GetHintText(Id, szBuf, sizeof(szBuf), htTooltip);
      }
   return string(szBuf);
}

void
JPButtonList::GetDesiredSize(TSize& size)
{
  size = (0,0);
  if (gadgetWindow)
      {
      TSize gadgsize (0,0);
      int maxsizex = 0;
      int maxsizey = 0;
      for (TGadget* g = gadgetWindow->FirstGadget(); g; g = gadgetWindow->NextGadget(*g))
        {
        gadgsize = (0,0);
        JPFlatButtonTextGadget* butg = TYPESAFE_DOWNCAST(g, JPFlatButtonTextGadget);
        if (butg)
           {
           butg->GetDefaultBitmapSize(gadgsize);
           maxsizex = max (maxsizex, gadgsize.cx);
           maxsizey = max (maxsizey, gadgsize.cy);
           }
        }
      if (maxsizey > 0)
         size.cy = maxsizey + 2;
      if (maxsizex > 0)
         size.cx = maxsizex + 2;
      }
}

void
JPButtonList::SetDesiredSize(TSize size)
{
   sizex = size.cx;
   sizey = size.cy;
}

void
JPButtonList::RebuildList()
{
   while (GetCount())
       DeleteString(0);
   StoreDataInListBox();
}

void
JPButtonList::ChangeGadgetWindow(TGadgetWindow* newgadgwin)
{
   if ((newgadgwin != gadgetWindow) && (GetHandle()))
      {
      gadgetWindow = newgadgwin;
      RebuildList();
      }
}

