/*******************************************************************************
*   Project BUTTON                                                             *
*   http://space.tin.it/computer/giparrel, joparrel@tin.it                     *
*   Copyright  1998. All Rights Reserved.                                     *
*                                                                              *
*   SUBSYSTEM:    button.apx Application                                       *
*   FILE:         jpcstbar.cpp                                                 *
*   AUTHOR:       Jo Parrello.                                                 *
*                                                                              *
*   OVERVIEW                                                                   *
*   ~~~~~~~~----                                                               *
*   Source file for implementation of JPCustomizeToolbar (TDialog).            *
*                                                                              *
*   Revision:   1.0  (May 15, 1998).                                           *
*                                                                              *
*   THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF          *
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO        *
*   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A             *
*   PARTICULAR PURPOSE.                                                        *
*******************************************************************************/
#include <owl/pch.h>

#include "buttonap.h"
#include "jpcstbar.h"


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(JPCustomizeToolbar, TDialog)
//{{JPCustomizeToolbarRSP_TBL_BEGIN}}
  EV_BN_CLICKED(IDC_BUTTON1, BNClicked),
  EV_BN_CLICKED(IDC_BUTTON2, BNClicked2),
  EV_BN_CLICKED(IDC_MOVEDOWN, BNMouseDnClicked),
  EV_BN_CLICKED(IDC_MOVEUP, BNMouseUpClicked),
  EV_CBN_SELCHANGE(IDC_COMBOBOX1, CBNSelchange),
  EV_BN_CLICKED(IDC_BUTTON3, BNClickedRestore),
//{{JPCustomizeToolbarRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{JPCustomizeToolbar Implementation}}


static JPCustomizeToolbarXfer JPCustomizeToolbarData;

JPCustomizeToolbar::JPCustomizeToolbar(TWindow* parent, TResId resId, TModule* module)
:
    TDialog(parent, resId, module)
{
//{{JPCustomizeToolbarXFER_USE}}
  AvailList = 0;
  ActiveList = 0;
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if ((theApp) && ((theApp->GadgetAvailableList) && (theApp->ToolbarList.GetItemsInContainer())))
        {
        gadgWin = theApp->ToolbarList[0];
        AvailList  = new JPButtonList(this, IDC_LISTBOX1, theApp->GadgetAvailableList);
        ActiveList = new JPButtonList(this, IDC_LISTBOX2, gadgWin);
        ToolbarBox = new TComboBox(this, IDC_COMBOBOX1, 50);
        }
//  SetTransferBuffer(&JPCustomizeToolbarData);
//{{JPCustomizeToolbarXFER_USE_END}}

  // INSERT>> Your constructor code here.

}


JPCustomizeToolbar::~JPCustomizeToolbar()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

}


void JPCustomizeToolbar::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.
  if ((AvailList) && (ActiveList))
     {
     TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
     if (theApp)
        {
        char buf[250];
        for (uint i = 0; i < theApp->ToolbarList.GetItemsInContainer(); i++)
            {
            TGadgetWindow* wind = theApp->ToolbarList[i];
            wind->GetWindowText(buf, sizeof(buf));
            if (strlen(buf) > 0)
            ToolbarBox->AddString(buf);
            }
        if (ToolbarBox->GetCount())
           ToolbarBox->SetSelIndex(0);
        }
     TSize leftpanesize = (0,0);
     TSize rightpanesize = (0,0);
     AvailList->GetDesiredSize(leftpanesize);
     ActiveList->GetDesiredSize(rightpanesize);
     if ((leftpanesize.cy > 0) || (rightpanesize.cy > 0))
        {          /*  If there is a button on one listbox    */
        if ((leftpanesize.cy > 0) && (rightpanesize.cy == 0))
           ActiveList->SetDesiredSize(leftpanesize);   /* right pane itmes must be sized as left pane ones */
        else
           if ((leftpanesize.cy == 0) && (rightpanesize.cy > 0))
              AvailList->SetDesiredSize(rightpanesize); /* left pane items must be sized as right pane ones */
        }
     else
        {     /*   left and right pane items are sized with default sizes    */
        ActiveList->SetDesiredSize(TSize(22,22));
        AvailList->SetDesiredSize(TSize(22,22));
        }
     }
  else
        {
        MessageBox("Unable to customize the toolbars.", "Customize Error", MB_OK | MB_ICONSTOP);
        Destroy(IDCANCEL);
        }
}

TGadget*
JPCustomizeToolbar::FindGadgetId(TGadgetWindow* gwin, int id, JPButtonList* list, int position)
{
if (id > 0)
   return gwin->GadgetWithId(id);
else
    {
    TGadget* gadgetret = NULL;
    TGadget* gt = gwin->FirstGadget();
    int maxvalue = list->GetCount();
    if (maxvalue > 0)
       {
       int curpos = 0;
       while (gt)
           {
           TGadget* gadg = gt;
           int id = gadg->GetId();
           if ((id == 0) && (curpos == position))
               gadgetret = gt;
           gt = gwin->NextGadget(*gadg);
           curpos++;
           }
       }
    return gadgetret;
    }
}

//
//  Add a button in Active Gadgets List
//
void JPCustomizeToolbar::BNClicked()
{
  // INSERT>> Your code here.
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp)
     {
     int selright = ActiveList->GetSelIndex();
     if ((selright < 0) && (ActiveList->GetCount() > 0))
        MessageBox("If you want to add a button to Right Pane, you must select an item on Right Pane.", "Customize Error", MB_OK);
     else
         {
         int selleft = AvailList->GetSelIndex();
         if ((selleft < 0) && (AvailList->GetCount() > 1))
            MessageBox("Please select an item on Left Pane.", "Customize Error", MB_OK);
         else
            {
            if ((selleft < 0) && (AvailList->GetCount() == 1))
                selleft = 0;       // select the separator gadget (default value)
            int valueleft = (int) AvailList->GetItemData(selleft);
            TGadget* gadgleft  = FindGadgetId(theApp->GadgetAvailableList, valueleft, AvailList, selleft);
            TGadget* gadgright = NULL;
            if (ActiveList->GetCount() > 0)
               {
               int valueright = (int) ActiveList->GetItemData(selright);
               gadgright = FindGadgetId(gadgWin, valueright, ActiveList, selright);
               }
            if (gadgleft)
               {
               if (theApp->InsertGadgetInToolbar(valueleft, gadgWin, TGadgetWindow::Before, gadgright))
                  {
                  if (selleft > 0)
                     {
                     AvailList->DeleteString(selleft);
                     AvailList->Invalidate();
                     if (selleft > (AvailList->GetCount() - 1))
                        selleft = AvailList->GetCount() - 1;
                     }
                  AvailList->SetSelIndex(selleft);
                  gadgWin->Invalidate(true);
                  gadgWin->LayoutSession();
                  ActiveList->InsertString((LPCTSTR) valueleft, selright);
                  ActiveList->Invalidate();
                  ActiveList->SetSelIndex(selright + 1);
                  }
               }
            }
         }
     }
}


//
//  Add a button in Available Gadgets List
//
void JPCustomizeToolbar::BNClicked2()
{
  // INSERT>> Your code here.
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp)
     {
     int selright = ActiveList->GetSelIndex();
     if (selright < 0)
        MessageBox("If you want to add a button to Left Pane, you must select an item on Right Pane.", "Customize Error", MB_OK);
     else
         {
         int selleft = AvailList->GetSelIndex();
         if ((selleft < 0) && (AvailList->GetCount() > 1))
            selleft = AvailList->GetCount() - 1;
         if ((selleft < 0) && (AvailList->GetCount() == 1))
             selleft = 0;       // select the separator gadget (default value)
         int valueleft  = (int) AvailList->GetItemData(selleft);
         int valueright = (int) ActiveList->GetItemData(selright);
         TGadget* gadgleft  = FindGadgetId(theApp->GadgetAvailableList, valueleft, AvailList, selleft);
         TGadget* gadgright = FindGadgetId(gadgWin, valueright, ActiveList, selright);
         if ((gadgleft) && (gadgright))
            {
            if (theApp->RemoveGadgetFromToolbar(gadgright, gadgWin, TGadgetWindow::Before, gadgleft))
               {
               if (valueright != 0)
                  {
                  if (selleft == 0)
                     {
                     selleft++;
                     AvailList->InsertString((LPCTSTR) valueright, selleft);
                     }
                  else
                     {
                     AvailList->InsertString((LPCTSTR) valueright, selleft);
                     selleft++;
                     }
                  AvailList->Invalidate();
                  }
               AvailList->SetSelIndex(selleft);
               gadgWin->Invalidate(true);
               gadgWin->LayoutSession();
               ActiveList->DeleteString(selright);
               ActiveList->Invalidate();
               if ((selright == 0) && (ActiveList->GetCount() > 0))
                  ActiveList->SetSelIndex(0);
               else
                  ActiveList->SetSelIndex(selright - 1);
               }
            }
         }
     }
}


void JPCustomizeToolbar::BNMouseDnClicked()
{
  // INSERT>> Your code here.
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp)
     {
     int selright = ActiveList->GetSelIndex();
     if (selright < 0)
        MessageBox("Select an item if you want to move it.", "Customization Error", MB_OK);
     else
        {
        if (ActiveList->GetCount() > 1)
           {
           int valueright = (int) ActiveList->GetItemData(selright);
           int valuenext  = (int) ActiveList->GetItemData(selright + 1);
           TGadget* gadgright = FindGadgetId(gadgWin, valueright, ActiveList, selright);
           TGadget* nextright = FindGadgetId(gadgWin, valuenext,  ActiveList, selright + 1);
           if ((gadgright) && (nextright))
              {
              TGadget* movedgadget = gadgWin->Remove(*gadgright);
              if (movedgadget)
                 {
                 ActiveList->DeleteString(selright);
                 ActiveList->InsertString((LPCTSTR) valueright, selright + 1);
                 ActiveList->SetSelIndex(selright + 1);
                 ActiveList->Invalidate();
                 gadgWin->Insert(*movedgadget, TGadgetWindow::After, nextright);
                 gadgWin->Invalidate(true);
                 gadgWin->LayoutSession();
                 }
              }
           }
        }
     }
}


void JPCustomizeToolbar::BNMouseUpClicked()
{
  // INSERT>> Your code here.
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp)
     {
     int selright = ActiveList->GetSelIndex();
     if (selright < 0)
        MessageBox("Select an item if you want to move it.", "Customization Error", MB_OK);
     else
        {
        if ((ActiveList->GetCount() > 1) && (selright > 0))
           {
           int valueright   = (int) ActiveList->GetItemData(selright);
           int valuebefore  = (int) ActiveList->GetItemData(selright - 1);
           TGadget* gadgright = FindGadgetId(gadgWin, valueright, ActiveList, selright);
           TGadget* beforeright = FindGadgetId(gadgWin, valuebefore,  ActiveList, selright - 1);
           if ((gadgright) && (beforeright))
              {
              TGadget* movedgadget = gadgWin->Remove(*gadgright);
              if (movedgadget)
                 {
                 ActiveList->DeleteString(selright);
                 ActiveList->InsertString((LPCTSTR) valueright, selright - 1);
                 ActiveList->SetSelIndex(selright - 1);
                 ActiveList->Invalidate();
                 gadgWin->Insert(*movedgadget, TGadgetWindow::Before, beforeright);
                 gadgWin->Invalidate(true);
                 gadgWin->LayoutSession();
                 }
              }
           }
        }
     }
}


void JPCustomizeToolbar::CBNSelchange()
{
  // INSERT>> Your code here.
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp)
     {
     int selright = ToolbarBox->GetSelIndex();
     TGadgetWindow* wind = theApp->ToolbarList[selright];
     if (wind != gadgWin)
        {
        gadgWin = wind;
        ActiveList->ChangeGadgetWindow(wind);
        }
     }
}


void JPCustomizeToolbar::BNClickedRestore()
{
  // INSERT>> Your code here.
  TBUTTONApp* theApp = TYPESAFE_DOWNCAST(GetApplication(), TBUTTONApp);
  if (theApp)
     {
     theApp->ReloadToolbarData();
     AvailList->RebuildList();
     ActiveList->RebuildList();
     }
}

