/*************************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.                  *
*  Site   : http://space.tin.it/computer/giparrel                                    *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                         *
*                                                                                    *
*  Revision:   1.0  (May 15, 1998).                                                  *
*                                                                                    *
*  Definition of JPFlatButtonTextGadget class.                                       *
*                                                                                    *
*  Read JPFLBUTE.TXT for use and license.                                            *
*                                                                                    *
*  Only for OWL 5.02 and 5.01 users.                                                 *
*  OWL and ObjectWindows are trademarks of Borland International.                    *
*                                                                                    *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF                *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO               *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                    *
*  PARTICULAR PURPOSE.                                                               *
*************************************************************************************/
#include <owl/pch.h>

#if !defined(OWL_UIHELPER_H)
#include <owl/uihelper.h>
#endif

#include "jpflbute.h"


//{{JPFlatButtonTextGadget Implementation}}

// Constants for DrawTextEx function (Win16 targets).
#if defined(BI_PLAT_WIN16)
#define DT_END_ELLIPSIS     0x00008000
#define DT_MODIFYSTRING     0x00010000
#endif

JPFlatButtonTextGadget::JPFlatButtonTextGadget (const char far* text,
                                                JPDisplayType disptype,
                                                JPTextPos    pos,
                                                uint maxChars,
                                                uint numrows,
                                                TResId glyphResIdOrIndex,
                                                int id,
                                                TType type,
                                                bool enabled,
                                                TState state,
                                                bool sharedGlyph)
:
    TButtonGadget(glyphResIdOrIndex, id, type, enabled, state, sharedGlyph)
{
  // INSERT>> Your constructor code here.
   CurrentText = NULL;
   CurrentTextLen = NULL;
   SetText(text);
   DispType = DefaultDispType = disptype;
   TextPos  = DefaultTextPos  = pos;
   MaxChars = DefaultMaxChars = maxChars;
   NumRows  = DefaultNumRows  = max(1, numrows);
   RetrieveHotTrackInfo();
   StoredBorders = GetBorders();
   if (DispType == BitmapText)
      {
      TBorders bord;
      bord.Left = bord.Right = bord.Top = bord.Bottom = 1;
      Borders = bord;
      }
   isEnter = isStoredRect = false;
   if (State==Down)
   	SetFlatState(fsDown);
   else SetFlatState(fsNormal);
}


JPFlatButtonTextGadget::~JPFlatButtonTextGadget()
{
  // INSERT>> Your destructor code here.

}


void JPFlatButtonTextGadget::PaintBorder(TDC& dc)
{
   if (GetFlatState() == fsUp)
      {
      TRect rect(0,0,Bounds.Width(),Bounds.Height());
      #if defined(BI_PLAT_WIN16)
      TUIBorder border(rect, TUIBorder::TEdge(TUIBorder::WndRaised), TUIBorder::Rect | TUIBorder::Adjust);
      #elif defined(BI_PLAT_WIN32)
      TUIBorder border(rect, TUIBorder::TEdge(TUIBorder::RaisedInner), TUIBorder::Rect | TUIBorder::Adjust);
      #endif
      border.Paint(dc);
      }
   else if (GetFlatState() == fsDown)
      {
      BorderStyle = Recessed;
      TButtonGadget::PaintBorder(dc);
      }
}

void
JPFlatButtonTextGadget::SetFlatState(FlatState state, bool update)
{
switch (state)
	{
   case fsNormal:
      CurrentFlatState = fsNormal;
      break;

   case fsUp:
      if (GetEnabled())
         CurrentFlatState = fsUp;
      else return;
      break;

   case fsDown:
      if ((GetEnabled()) || (GetButtonState() == Down))
         CurrentFlatState = fsDown;
      else return;
      break;
   }
if (update)
	{
   InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
	Update();
   }
}

JPFlatButtonTextGadget::FlatState
JPFlatButtonTextGadget::GetFlatState() const
{
  return  CurrentFlatState;
}


void JPFlatButtonTextGadget::MouseEnter(uint modKeys, TPoint& point)
{
  TButtonGadget::MouseEnter(modKeys, point);

  // INSERT>> Your code here.
  isEnter = true;
  if ((GetButtonState()==Up) && (GetEnabled()))
  		SetFlatState(fsUp);
  else if (GetFlatState()==fsDown)
          Invalidate();
}


void JPFlatButtonTextGadget::MouseLeave(uint modKeys, TPoint& point)
{
  TButtonGadget::MouseLeave(modKeys, point);

  // INSERT>> Your code here.
  isEnter = false;
  if ((State!=Down) && (GetEnabled()))
      SetFlatState(fsNormal);
  else if (GetFlatState()==fsDown)
          Invalidate();
}


void JPFlatButtonTextGadget::BeginPressed(TPoint& p)
{
  SetFlatState(fsDown , false);
  if (DispType != Bitmap)
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
  if (isEnter == false)
     {
     isEnter = Pressed = true;
     TButtonGadget::MouseEnter(0, p);
     }
  TButtonGadget::BeginPressed(p);
}


void JPFlatButtonTextGadget::CancelPressed(TPoint& p)
{
  bool oldpressed = Pressed;
  if ((State==Up) && (oldpressed))
  		{
      bool hit = PtIn(p);
      if ((hit) && GetEnabled())
  				SetFlatState(fsUp);
      else
         {
         isEnter = Pressed = false;
  			SetFlatState(fsNormal);
         TButtonGadget::MouseLeave(0, p);
         }
      }
  TButtonGadget::CancelPressed(p);
}


void
JPFlatButtonTextGadget::SetButtonState(TState newState)
{
  TButtonGadget::SetButtonState(newState);
   if (State==Down)
      SetFlatState(fsDown);
   else
      SetFlatState(fsNormal);
}

bool
JPFlatButtonTextGadget::IdleAction(long idleCount)
{
  if (idleCount == 0)
    {
    if (GetEnabled()==false)
    	{
    	if ((GetFlatState()!=fsNormal) && (GetButtonState()==Up) && (GetButtonType()==Command))
    	   SetFlatState(fsNormal);
      }
    else
    	{
      if (isEnter==true)
      	{
      	if ((GetFlatState()==fsNormal) && (GetButtonState()==Up))
            SetFlatState(fsUp);
         else if ((GetFlatState()==fsUp) && (GetButtonState()==Down))
         	SetFlatState(fsDown);
         }
      }
    }
  return TButtonGadget::IdleAction(idleCount);
}

void
JPFlatButtonTextGadget::SetEnabled(bool enabled)
{
  SetTextFromResource();        // retrieve the text string when possible
  if (enabled != GetEnabled())  // this "if" is to avoid a continuous flickering
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true); // erase shadow of text
  TButtonGadget::SetEnabled(enabled);
}

/*******************************************************************************
*   Called when system colors change.                                          *
*   Under Windows 98 retrieve the system's hot-track colors (if available).    *
*******************************************************************************/
void
JPFlatButtonTextGadget::SysColorChange()
{
  TButtonGadget::SysColorChange();

  RetrieveHotTrackInfo();
  InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
  Update();
}

/*******************************************************************************
*   This method returns the size of the simple gadget if bitmap isn't needed.  *
*******************************************************************************/
void JPFlatButtonTextGadget::GetDesiredSize(TSize& size)
{
  if (DispType == Text)
     TGadget::GetDesiredSize(size);
  else
     TButtonGadget::GetDesiredSize(size);
}

/*******************************************************************************
*   This method paints the text before and the default bitmap after.           *
*******************************************************************************/
void JPFlatButtonTextGadget::Paint(TDC& dc)
{
    uint fstate = GetFlatState();
    uint nstate = GetButtonState();
    if ((fstate != fsUp) && (!Pressed))
    	{
      if ((nstate == Down) && (fstate != fsDown))
      	{
         SetFlatState(fsDown, false);
         InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
         }
      else if ((nstate == Up) && (fstate != fsNormal))
         {
         SetFlatState(fsNormal, false);
   		InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
         }
      }
  if (DispType != Bitmap)
     {
     if (Window)
        {
        TFont* font = &(Window->GetFont());
        if (font)
           dc.SelectObject(*font);
        }
     TRect rect,rect1;
     rect1 = CurrentTextRect;
     uint offset = (Pressed || State == Down) ? 1 : 0;
     rect1.Offset(offset, offset);
     /*GetInnerRect(rect);
     TBorders bord = GetBorders();
     uint offset = (Pressed || State == Down) ? 1 : 0;
     int numwidth = (DispType == Text) ? 0 : StoredRect.Width();
     if (bord.Left == bord.Right)
        rect1.Offset(bord.Left + offset, offset);
     if (rect1.Width() >= Bounds.Width())
         rect1.right = rect1.left + Bounds.Width() - 1;

     switch (TextPos) {
       case Left:
         if ( (rect1.Width() + numwidth) < Bounds.Width() )
            rect1.Offset( (Bounds.Width() - numwidth - rect1.Width() ) / 2, 0);
         rect1.Offset(0, (Bounds.Height() - rect1.Height()) / 2);
         break;

       case Right:
         if ( (rect1.Width() + numwidth) < Bounds.Width() )
            rect1.Offset(numwidth + (Bounds.Width() - numwidth - rect1.Width()) / 2,0);
         rect1.Offset(0, (Bounds.Height() - rect1.Height()) / 2);
         break;

       case Top:
         if (rect1.Width() < Bounds.Width())
            rect1.Offset(((Bounds.Width() - rect1.Width()) / 2), 0);
         rect1.Offset(0, bord.Top + ((rect.top - rect1.Height()) / 2));
         break;

       case Bottom:
         if (rect1.Width() < Bounds.Width())
            rect1.Offset(((Bounds.Width() - rect1.Width()) / 2), 0);
         rect1.Offset(0, rect.bottom + ((Bounds.Height()-rect.bottom-rect1.Height()) / 2));
         break;
       }*/
     //dc.SetBkColor(TColor::Sys3dHilight);
     dc.SetBkColor(TColor::Sys3dFace);
     dc.SetBkMode(OPAQUE);
     if (GetEnabled() == false)
        {
        rect1.Offset(1,1);
        dc.SetTextColor(TColor::Sys3dHilight);
        if (NumRows > 1)
           DrawTextEx(dc, CurrentText, CurrentTextLen, rect1, DT_CENTER | DT_END_ELLIPSIS | DT_WORDBREAK);
        else
           DrawTextEx(dc, CurrentText, CurrentTextLen, rect1, DT_LEFT | DT_SINGLELINE | DT_END_ELLIPSIS);
        rect1.Offset(-1,-1);
        dc.SetBkMode(TRANSPARENT);
        }
     if ((GetFlatState()==fsNormal) || (HotTrack == false) || (isEnter == false))
        dc.SetTextColor(GetEnabled() ? TColor::SysBtnText : TColor::SysGrayText);
     else
        dc.SetTextColor(GetEnabled() ? TColor(GetHotTrackColor()) : TColor::SysGrayText);
     if (NumRows > 1)
         DrawTextEx(dc, CurrentText, CurrentTextLen, rect1, DT_CENTER | DT_END_ELLIPSIS | DT_WORDBREAK);
     else
         DrawTextEx(dc, CurrentText, CurrentTextLen, rect1, DT_LEFT | DT_SINGLELINE | DT_END_ELLIPSIS);
     dc.RestoreFont();
   }

  if (DispType != Text)
     {
     if ((DispType == BitmapText) && (Pressed == false) && (GetFlatState()==fsDown))
        {
        Pressed = true;
        TButtonGadget::Paint(dc);
        Pressed = false;
        }
     else
        TButtonGadget::Paint(dc);
     }
  else
     {
     BorderStyle = (Pressed || State == Down) ? ButtonDn : ButtonUp;
     PaintBorder(dc);
     }
}

/*******************************************************************************
*   This method sets the current text.                                         *
*******************************************************************************/
void JPFlatButtonTextGadget::SetText(const char far* text)
{
  if (text && CurrentText && strcmp(text, CurrentText) == 0)
    return;

  delete[] CurrentText;

  if (text)
    {
    CurrentText = strnewdup(text);
    CurrentTextLen = strlen(CurrentText);
    }
  else
    {
    if (SetTextFromResource() == false)
       {
       CurrentText = NULL;
       CurrentTextLen = NULL;
       }
    }

  if (Window)
    Window->GadgetChangedSize(*this);

  Invalidate();
}

/*******************************************************************************
*   This method retrieves the current text.                                    *
*******************************************************************************/
const char far* JPFlatButtonTextGadget::GetText() const
{
  return CurrentText;
}

/*******************************************************************************
*   This method tries to retrieve the resource string as the text.             *
*******************************************************************************/
bool JPFlatButtonTextGadget::SetTextFromResource()
{
    if ((Window) && (Window->GetHandle()) && (!CurrentText))
       {
       string text = LoadInternalString(GetId());
       if (text.length())
          {
          delete [] CurrentText;
          CurrentText = strnewdup(text.c_str());
          CurrentTextLen = strlen(CurrentText);
          return true;
          }
       }
    return false;
}

/*******************************************************************************
*   This method is called only when button has been created.                   *
*******************************************************************************/
void JPFlatButtonTextGadget::Created()
{
    TButtonGadget::Created();

    if (!isStoredRect)
       {
       // store the default size of the bitmap (used by other methods).
       TSize size;
       GetDefaultBitmapSize(size);
       StoredRect = TRect(TPoint(0,0),size);
       isStoredRect = true;
       }

    // if there is no text, tries to retrieve the resource string.
    SetTextFromResource();

    // dispose the bitmap and the text on the gadget.
    DisposeUnitsOnGadget();
}

/*******************************************************************************
*   This method disposes the bitmap and the text on the gadget.                *
*******************************************************************************/
void JPFlatButtonTextGadget::DisposeUnitsOnGadget()
{
  if ((Window) && (Window->GetHandle()))
      {
      TClientDC dc(Window->GetHandle());
      TFont* font = &(Window->GetFont());
      if (font)
          dc.SelectObject(*font);
      int MaxWidth  = (MaxChars * font->GetMaxWidth());
      int maxwidth  = MaxWidth;
      int MaxHeight = font->GetHeight();
//      TRect rect(0, 0, MaxWidth, MaxHeight * NumRows);
      if ((DispType != Text) /*&& (rect.right > StoredRect.Width())*/ && ((TextPos == Left) || (TextPos == Right)))
         MaxWidth = MaxWidth - MulDiv(StoredRect.Width(), 72, dc.GetDeviceCaps(LOGPIXELSY));
//         rect.right = rect.right - StoredRect.Width();
      TRect rect(0, 0, MaxWidth, MaxHeight * NumRows);
      if (NumRows > 1)
         dc.DrawText(CurrentText, -1, rect, DT_CALCRECT | DT_WORDBREAK);
      else
         dc.DrawText(CurrentText, -1, rect, DT_CALCRECT | DT_SINGLELINE);
      dc.RestoreFont();
      CurrentTextRect = rect;
      int numrow = MaxHeight * NumRows;
      if (((TextPos == Left) || (TextPos == Right)) && (DispType == BitmapText))
         {
         if (numrow < StoredRect.Height())
            rect.bottom = 0;
         else rect.bottom = numrow - StoredRect.Height();
         }
      else
         rect.bottom = numrow;
      if (CurrentTextRect.right > MaxWidth)
         CurrentTextRect.right = MaxWidth;
      if (CurrentTextRect.bottom > numrow)
         CurrentTextRect.bottom = numrow;
      MaxWidth = maxwidth;
      
      if (DispType != Bitmap)
         {
         if (((TextPos == Top) || (TextPos == Bottom)) && (DispType == BitmapText))
             MaxWidth = MaxWidth - StoredRect.Width();
         TMargins mrect;
         mrect = Margins;
         TBorders bord;
         if (DispType == BitmapText)
            bord.Left = bord.Right = bord.Top = bord.Bottom = 1;
         else bord = StoredBorders;
         Borders = bord;

         if (DispType == BitmapText)
            {
            switch (TextPos) {
             case Left:
               mrect.Left = MaxWidth / 2;
               mrect.Top = ((rect.bottom * 8 - 4) / MaxHeight / 2) + bord.Top;
               mrect.Bottom = ((rect.bottom * 8 - 4) / MaxHeight / 2) + bord.Bottom;
               mrect.Right = 0;
               break;

             case Right:
               mrect.Right = MaxWidth / 2;
               mrect.Top = ((rect.bottom * 8 - 4) / MaxHeight / 2) + bord.Top;
               mrect.Bottom = ((rect.bottom * 8 - 4) / MaxHeight / 2) + bord.Bottom;
               mrect.Left = 0;
               break;

             case Top:
               mrect.Left = mrect.Right = MaxWidth / 4;
               mrect.Top = ((rect.bottom * 8 - 4) / MaxHeight) + bord.Top + bord.Bottom;
               mrect.Bottom = 0;
               break;

             case Bottom:
               mrect.Left = mrect.Right = (MaxWidth / 4) + bord.Left +  bord.Right;
               mrect.Bottom = ((rect.bottom * 8 - 4) / MaxHeight) + bord.Top + bord.Bottom;
               mrect.Top = 0;
               break;
             }
            }
            else
               {
               mrect.Left = mrect.Right = (MaxWidth / 4) + bord.Left;
               mrect.Bottom = ((rect.bottom * 8) / MaxHeight) + bord.Top + bord.Bottom;
               mrect.Top = 0;
               }

         SetMargins(mrect);
         TRect  rectBounds = Bounds;
         SetBounds(rectBounds);
         TSize sz;
         Window->GetDesiredSize(sz);

         // If the parent window is a TGadgetWindow, we must resize its client area
         // (we don't need this code if the parent window is a Dockable Window).
         if ((Window->Attr.W != sz.cx) || (Window->Attr.H != sz.cy))
           {
           Window->Attr.W = sz.cx;
           Window->Attr.H = sz.cy;
           TWindow* parent= Window->GetParentO();
           TDecoratedFrame* frame= parent ?
                                   TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
           while (parent && !frame) {
             parent = parent->GetParentO();
             if (parent)
               frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
             }
           if (frame)
              {
              TLayoutMetrics  metrics;
              frame->GetChildLayoutMetrics(*Window, metrics);
              frame->SetChildLayoutMetrics(*Window, metrics);
              frame->Layout();
              }
           }
         }
     TRect rect1;
     GetInnerRect(rect);
     rect1 = CurrentTextRect;
     TBorders border = GetBorders();
     int numwidth = (DispType == Text) ? 0 : StoredRect.Width();
     if (border.Left == border.Right)
        rect1.Offset(border.Left, 0);
     if (rect1.Width() >= Bounds.Width())
         rect1.right = rect1.left + Bounds.Width() - 1;

     switch (TextPos) {
       case Left:
         if ( (rect1.Width() + numwidth) < Bounds.Width() )
            rect1.Offset( (Bounds.Width() - numwidth - rect1.Width() ) / 2, 0);
         rect1.Offset(0, (Bounds.Height() - rect1.Height()) / 2);
         break;

       case Right:
         if ( (rect1.Width() + numwidth) < Bounds.Width() )
            rect1.Offset(numwidth + (Bounds.Width() - numwidth - rect1.Width()) / 2,0);
         rect1.Offset(0, (Bounds.Height() - rect1.Height()) / 2);
         break;

       case Top:
         if (rect1.Width() < Bounds.Width())
            rect1.Offset(((Bounds.Width() - rect1.Width()) / 2), 0);
         rect1.Offset(0, border.Top + ((rect.top - rect1.Height()) / 2));
         break;

       case Bottom:
         if (rect1.Width() < Bounds.Width())
            rect1.Offset(((Bounds.Width() - rect1.Width()) / 2), 0);
         rect1.Offset(0, rect.bottom + ((Bounds.Height()-rect.bottom-rect1.Height()) / 2));
         break;
       }
     CurrentTextRect = rect1;
     }
}

/*******************************************************************************
*   This method allows to retrieve the resource string for this button.        *
*******************************************************************************/
string JPFlatButtonTextGadget::LoadInternalString(uint itemId)
{
   char tempstr[256], szBuf[256];
   char* ptr = NULL, c = '\n';
   strcpy(szBuf,"");
   strcpy(tempstr,"");
   int len = Window->GetModule()->LoadString(itemId, tempstr, sizeof(szBuf));
   if (len > 0)
      ptr = strchr(tempstr, c);
   if (ptr)
      strncpy(szBuf, ptr + 1, min(len, (int) sizeof(szBuf)));
   else
      {
      TWindow* parent= Window->GetParentO();
      TDecoratedFrame* frame= parent ? TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
      while (parent && !frame)
      {
        parent = parent->GetParentO();
        if (parent)
          frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
      }
      if (frame)
          frame->GetHintText(itemId, szBuf, sizeof(szBuf), htTooltip);
      }
   return string(szBuf);
}

#if defined(BI_PLAT_WIN32)
/*******************************************************************************
*   This method allows the use of DrawTextEx (not defined by OWL).             *
*******************************************************************************/
int
JPFlatButtonTextGadget::DrawTextEx(TDC& dc, const char far* str, int count,
                                   const TRect& rect, uint16 format)
{
  DRAWTEXTPARAMS param;
  param.cbSize = sizeof (DRAWTEXTPARAMS);
  param.iTabLength = param.iLeftMargin = param.iRightMargin = 0;
  param.uiLengthDrawn = count;
  return ::DrawTextEx(dc, (LPTSTR) str, count, (RECT*)&rect, format, &param);
}
#elif defined(BI_PLAT_WIN16)
/*******************************************************************************
*   This method emulates DrawTextEx on Win16 platforms.                        *
*   It emulates DT_END_ELLIPSIS flag too, but is limited to draw.              *
*   Don't use this method with DT_CALCRECT or DT_MODIFYSTRING.                 *
*******************************************************************************/
int
JPFlatButtonTextGadget::DrawTextEx(TDC& dc, const char far* str, int count,
                                   const TRect& rect, uint16 format)
{
   int retValue = 0;
   bool isDrawn = false;
   bool win32Bit = true;
   DWORD dwVersion = ::GetVersion();
   if ((LOBYTE(LOWORD(dwVersion)) == 3) && (HIBYTE(LOWORD(dwVersion)) < 95))
      win32Bit = false;
  if (((format & DT_END_ELLIPSIS) == DT_END_ELLIPSIS) && (win32Bit == false))
     {
     format &= ~DT_END_ELLIPSIS;
     TRect newrect = rect;
     dc.DrawText(str, count, newrect, DT_CALCRECT | format);
     if (rect.Contains(newrect) == false)
        {
        char buf[256];
        char *ellipse = "...";
        int buflen;
        for (int i = strlen(str); i--; i == 0)
           {
           strncpy(buf, str, i + 1);
           buf[i + 1] = '\0';
           strcat(buf, ellipse);
           buflen = strlen(buf);
           newrect = rect;
           dc.DrawText(buf, buflen, newrect, DT_CALCRECT | format);
           if (rect.Contains(newrect) == true)
               {
               retValue = dc.DrawText(buf, buflen, rect, format);
               isDrawn = true;
               break;
               }
           }
        }
     }
  if (isDrawn == false)
     retValue = dc.DrawText(str, count, rect, format);
  return retValue;
}
#endif

/*******************************************************************************
*   This method retrieve informations about Hot-Tracking feature.              *
*******************************************************************************/
void
JPFlatButtonTextGadget::RetrieveHotTrackInfo()
{
   TColor hotCol = TColor(0, 0, 255);  // Default Hottrack color is Blue
   bool   hotTrack = true;             // Hot-Tracking is enabled on default
#if defined(BI_PLAT_WIN32)
#ifndef COLOR_HOTLIGHT
#define COLOR_HOTLIGHT  26
#endif
#ifndef SPI_GETHOTTRACKING
#define SPI_GETHOTTRACKING 0x100E
#endif
   OSVERSIONINFO os;
   os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   ::GetVersionEx(&os);
   if ((os.dwMinorVersion > 0) && (os.dwMajorVersion >= 4))
      {
      // if in Windows 98 or higher retrieves the current values for Hot-Tracking
      ::SystemParametersInfo(SPI_GETHOTTRACKING, NULL, &hotTrack, NULL);
      hotCol = TColor(::GetSysColor(COLOR_HOTLIGHT));
      }
#endif
   SetHotTrackColor(hotCol.Red(), hotCol.Green(), hotCol.Blue());
   HotTrack = hotTrack;
}

/*******************************************************************************
*   This method sets the actual Hot Track color for the text.                  *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetHotTrackColor(uint8 r, uint8 g, uint8 b)
{
    HotTrackColor = TRgbTriple(r,g,b);

    Invalidate();
    Update();
}

/*******************************************************************************
*   This method returns the actual Hot Track color for the text.               *
*******************************************************************************/
COLORREF
JPFlatButtonTextGadget::GetHotTrackColor()
{
  return HotTrackColor.GetValue();
}

/*******************************************************************************
*   This method sets the actual boolean Hot Track value for the text.          *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetHotTrack(bool track)
{
    HotTrack = track;

    Invalidate();
    Update();
}

/*******************************************************************************
*   This method returns the actual boolean Hot Track value for the text.       *
*******************************************************************************/
bool
JPFlatButtonTextGadget::GetHotTrack()
{
  return HotTrack;
}

/*******************************************************************************
*    Set the display type for this button (Text, Bitmap, BitmapText).          *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDisplayType(JPDisplayType disp)
{
  if (disp != DispType)
     {
     DispType = disp;
     DisposeUnitsOnGadget();
     if (DispType == Bitmap)
        {
        Borders = StoredBorders;
        TMargins mrect;
        SetMargins(mrect);
        }
     Invalidate();
     }
}

/*******************************************************************************
*    Returns the display type for this button (Text, Bitmap, BitmapText).      *
*******************************************************************************/
JPFlatButtonTextGadget::JPDisplayType
JPFlatButtonTextGadget::GetDisplayType() const
{
  return  DispType;
}

/*******************************************************************************
*    Set the text position for this button (Left, Right, Top, Bottom).         *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetTextPosition(JPTextPos pos)
{
  if (pos != TextPos)
     {
     TextPos = pos;
     DisposeUnitsOnGadget();
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
     Update();
     }
}

/*******************************************************************************
*    Returns the text position for this button (Left, Right, Top, Bottom).     *
*******************************************************************************/
JPFlatButtonTextGadget::JPTextPos
JPFlatButtonTextGadget::GetTextPosition() const
{
  return  TextPos;
}

/*******************************************************************************
*    Set the text number of rows for this button.                              *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetTextRows(uint rows)
{
  if (rows != NumRows)
     {
     NumRows = max(1, rows);
     DisposeUnitsOnGadget();
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
     Update();
     }
}

/*******************************************************************************
*    Returns the text number of rows for this button.                          *
*******************************************************************************/
uint
JPFlatButtonTextGadget::GetTextRows() const
{
  return  NumRows;
}

/*******************************************************************************
*    Set the max text number of chars for this button.                         *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetMaxChars(uint maxchars)
{
  if (maxchars != MaxChars)
     {
     MaxChars = maxchars;
     DisposeUnitsOnGadget();
     InvalidateRect(TRect(0, 0, Bounds.Width(), Bounds.Height()), true);
     Update();
     }
}

/*******************************************************************************
*    Returns the max text number of chars for this button.                     *
*******************************************************************************/
uint
JPFlatButtonTextGadget::GetMaxChars() const
{
  return  MaxChars;
}

/*******************************************************************************
*    Set the default display type (as declared on constructor).                *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultDisplayType()
{
   SetDisplayType(DefaultDispType);
}

/*******************************************************************************
*    Set the default text position (as declared on constructor).               *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultTextPosition()
{
   SetTextPosition(DefaultTextPos);
}

/*******************************************************************************
*    Set the default text number of rows (as declared on constructor).         *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultTextRows()
{
   SetTextRows(DefaultNumRows);
}

/*******************************************************************************
*    Set the default max text number of chars (as declared on constructor).    *
*******************************************************************************/
void
JPFlatButtonTextGadget::SetDefaultMaxChars()
{
   SetMaxChars(DefaultMaxChars);
}

/*******************************************************************************
*   This method returns the size of the default bitmap.                        *
*******************************************************************************/
void JPFlatButtonTextGadget::GetDefaultBitmapSize(TSize& size)
{
   TDib* dib = GetGlyphDib();
   size.cx = dib->Width();
   size.cy = dib->Height();
}

/*******************************************************************************
*   This method returns the default glyph dib.                                 *
*******************************************************************************/
TDib* JPFlatButtonTextGadget::GetDefaultGlyphDib()
{
   return GetGlyphDib();
}

