/*************************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.                  *
*  Site   : http://space.tin.it/computer/giparrel                                    *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                         *
*                                                                                    *
*  Revision:   1.0  (May 15, 1998).                                                  *
*                                                                                    *
*  Definition of JPFlatButtonTextGadget class.                                       *
*                                                                                    *
*  Read JPFLBUTE.TXT for use and license.                                            *
*                                                                                    *
*  Only for OWL 5.02 and 5.01 users.                                                 *
*  OWL and ObjectWindows are trademarks of Borland International.                    *
*                                                                                    *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF                *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO               *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                    *
*  PARTICULAR PURPOSE.                                                               *
*************************************************************************************/
#if !defined(jpfltbtg_h)              // Sentry, use file only if it's not already included.
#define jpfltbtg_h

#if !defined(OWL_BUTTONGA_H)
#include <owl/buttonga.h>
#endif
#if !defined(OWL_GADGETWI_H)
#include <owl/gadgetwi.h>
#endif

//{{TButtonGadget = JPFlatButtonTextGadget}}
class JPFlatButtonTextGadget : public TButtonGadget {
  public:
    enum JPDisplayType {
         Bitmap,
         Text,
         BitmapText,
         };

    enum JPTextPos {
         Left,
         Right,
         Top,
         Bottom,
         };

    JPFlatButtonTextGadget(const char far* text,
                           JPDisplayType disptype,
                           JPTextPos    pos,
                           uint maxChars,
                           uint numrows,
                           TResId glyphResIdOrIndex,
                           int id,
                           TType type = Command,
                           bool enabled = false,
                           TState state = Up,
                           bool sharedGlyph = false);
    virtual ~JPFlatButtonTextGadget();

//{{JPFlatButtonTextGadgetVIRTUAL_BEGIN}}
  protected:
    enum FlatState {
    	fsNormal,
      fsUp,
      fsDown,
    };
    char far*     CurrentText;            // new'd copy of the text for this gadget
    uint          CurrentTextLen;         // strlen of the above text
    TRect         CurrentTextRect;        // Rectangle where the text will appear
    TRect         StoredRect;             // the bitmap's rect
    TColor        HotTrackColor;          // Hot Track Color for Text
    bool          HotTrack;               // Enable/Disable Hot Track for Text
    TBorders      StoredBorders;
    JPDisplayType DispType, DefaultDispType;
    JPTextPos     TextPos,  DefaultTextPos;
    uint          MaxChars, DefaultMaxChars;
    uint          NumRows,  DefaultNumRows;
    FlatState     CurrentFlatState;
    bool          isEnter, isStoredRect;
	 void 			SetFlatState(FlatState state, bool update=true);
    FlatState     GetFlatState() const;
    void Paint(TDC& dc);
    void Created();
    void DisposeUnitsOnGadget();
    bool SetTextFromResource();
    int  DrawTextEx(TDC& dc, const char far* str, int count, const TRect& rect, uint16 format);
    void RetrieveHotTrackInfo();

  public:
    // These allow to change the values at run-time.
    virtual void SetMaxChars(uint maxchars);
    virtual uint GetMaxChars() const;
    virtual void SetTextRows(uint rows);
    virtual uint GetTextRows() const;
    virtual void SetDisplayType(JPDisplayType disp);
    virtual JPDisplayType GetDisplayType() const;
    virtual void SetTextPosition(JPTextPos pos);
    virtual JPTextPos GetTextPosition() const;

    // Set the default values for this button.
    virtual void SetDefaultDisplayType();
    virtual void SetDefaultTextPosition();
    virtual void SetDefaultTextRows();
    virtual void SetDefaultMaxChars();

    //  Hot-Tracking feature.
    virtual  void SetHotTrackColor(uint8 r, uint8 g, uint8 b);
    COLORREF GetHotTrackColor();
    virtual void SetHotTrack(bool track);
    virtual bool GetHotTrack();

    // Methods for the text.
    virtual void SetText(const char far* text);
    virtual const char far* GetText() const;

    // derived methods.
    virtual string LoadInternalString(uint itemId);
    virtual void  GetDefaultBitmapSize(TSize& size);
    virtual TDib* GetDefaultGlyphDib();
    virtual void  PaintBorder(TDC& dc);
    virtual void  MouseEnter(uint modKeys, TPoint& point);
    virtual void  MouseLeave(uint modKeys, TPoint& point);
    virtual void  BeginPressed(TPoint& p);
    virtual void  CancelPressed(TPoint& p);
    virtual void  SetButtonState(TState newState);
    virtual bool  IdleAction(long idleCount);
    virtual void  GetDesiredSize(TSize& size);
    virtual void  SysColorChange();
    virtual void  SetEnabled(bool enabled);

//{{JPFlatButtonTextGadgetVIRTUAL_END}}
};    //{{JPFlatButtonTextGadget}}

#endif  // jpfltbtg_h sentry.

