/*************************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.                  *
*  Site   : http://space.tin.it/computer/giparrel                                    *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                         *
*                                                                                    *
*  Revision:   1.0  (May 15, 1998).                                                  *
*                                                                                    *
*  Definition of JPFlatMenuButtonTextGadget class.                                   *
*                                                                                    *
*  Read JPFLMENU.TXT for use and license.                                            *
*                                                                                    *
*  Only for OWL 5.02 and 5.01 users.                                                 *
*  OWL and ObjectWindows are trademarks of Borland International.                    *
*                                                                                    *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF                *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO               *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                    *
*  PARTICULAR PURPOSE.                                                               *
*************************************************************************************/
#include <owl/pch.h>

#include "jpflmenu.h"

#if defined(BI_PLAT_WIN16)
#define TPM_TOPALIGN        0x0000L
#define TPM_VCENTERALIGN    0x0010L
#define TPM_BOTTOMALIGN     0x0020L

#define TPM_HORIZONTAL      0x0000L
#define TPM_VERTICAL        0x0040L
#define TPM_NONOTIFY        0x0080L
#define TPM_RETURNCMD       0x0100L

  typedef struct tagTPMPARAMS
   {
       UINT    cbSize;
       RECT    rcExclude;
   }   TPMPARAMS;
  typedef TPMPARAMS FAR *LPTPMPARAMS;
#endif


JPFlatMenuButtonTextGadget::JPFlatMenuButtonTextGadget(
                                                TMenu& menu,
                                                const char far* text,
                                                JPDisplayType disptype,
                                                JPTextPos    pos,
                                                uint maxChars,
                                                uint numrows,
                                                TResId glyphResIdOrIndex,
                                                int id,
                                                TType type,
                                                bool enabled,
                                                TState state,
                                                bool sharedGlyph)
:
    JPFlatButtonTextGadget(text, disptype, pos, maxChars, numrows, glyphResIdOrIndex, id, type, enabled, state, sharedGlyph)
{
  GadgetPopupMenu = new TPopupMenu(menu);
}


JPFlatMenuButtonTextGadget::~JPFlatMenuButtonTextGadget()
{
  delete GadgetPopupMenu;
}


void JPFlatMenuButtonTextGadget::BeginPressed(TPoint& p)
{
  HWND ParentWnd;

  JPFlatButtonTextGadget::BeginPressed(p);

   TWindow* parent= Window->GetParentO();
   TDecoratedFrame* frame= parent ?
                           TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
   while (parent && !frame)
   {
     parent = parent->GetParentO();
     if (parent)
       frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
   }
   if (frame)
   	ParentWnd = *frame;
   else ParentWnd = NULL;
  if (ParentWnd)
     {
  		if (Window->GetHintMode() == TGadgetWindow::EnterHints)
    		Window->SetHintCommand(-1);
      TRect rect = GetBounds();
      TPoint p2(rect.TopLeft());
      Window->ClientToScreen(p2);
      rect.right = p2.x + (rect.right - rect.left);
      rect.bottom = p2.y + (rect.bottom - rect.top);
      rect.left = p2.x;
      rect.top = p2.y;
      p2.y = rect.bottom;
      #if defined(BI_PLAT_WIN16)
      if (TrackPopupMenuEx(TPM_VERTICAL, p2, ParentWnd, &rect) == false)
          GadgetPopupMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, p2, 0, ParentWnd);
      #elif defined(BI_PLAT_WIN32)
      TrackPopupMenuEx(TPM_VERTICAL, p2, ParentWnd, &rect);
      #endif
      CancelPressed(p);
      LButtonUp(0,p);
      }
}

bool JPFlatMenuButtonTextGadget::TrackPopupMenuEx(uint flags, const TPoint& point, HWND wnd, const TRect* rect)
{
  TPMPARAMS tpm;

  PRECONDITION(GadgetPopupMenu->GetHandle());
  tpm.cbSize=sizeof(TPMPARAMS);
  tpm.rcExclude=*rect;

#if defined(BI_PLAT_WIN16)
  typedef BOOL (WINAPI *FP)(HMENU, UINT, int, int, HWND, LPTPMPARAMS);

  static FP fp = 0;
  if (!fp) {
    fp = (FP)GetProcAddress(GetModuleHandle("USER"), "TrackPopupMenuEx");
  }
  if (fp)
    return fp(GadgetPopupMenu->GetHandle(), flags, point.x, point.y, wnd, &tpm) != 0;
  return false;

#elif defined(BI_PLAT_WIN32)
  return ::TrackPopupMenuEx(GadgetPopupMenu->GetHandle(), flags, point.x, point.y, wnd, &tpm);
#endif
}

