/*************************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.                  *
*  Site   : http://space.tin.it/computer/giparrel                                    *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                         *
*                                                                                    *
*  Revision:   1.0  (May 15, 1998).                                                  *
*                                                                                    *
*  Definition of JPFlatComboBox class.                                               *
*                                                                                    *
*  Read JPFLTBOX.TXT for use and license.                                            *
*                                                                                    *
*  Only for OWL 5.02 and 5.01 users.                                                 *
*  OWL and ObjectWindows are trademarks of Borland International.                    *
*                                                                                    *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF                *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO               *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                    *
*  PARTICULAR PURPOSE.                                                               *
*************************************************************************************/
#include <owl/pch.h>

#if !defined(OWL_TOOLTIP_H)
#include <owl/tooltip.h>
#endif
#if !defined(OWL_GADGETWI_H)
#include <owl/gadgetwi.h>
#endif

#include "jpfltbox.h"


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(JPFlatComboBox, TComboBox)
//{{JPFlatComboBoxRSP_TBL_BEGIN}}
  EV_WM_CTLCOLOR,
  EV_WM_PAINT,
  EV_WM_LBUTTONDOWN,
  EV_WM_LBUTTONUP,
  EV_WM_ERASEBKGND,
  EV_WM_NCCALCSIZE,
//  EV_WM_NCHITTEST,
  EV_WM_MOUSEMOVE,
//{{JPFlatComboBoxRSP_TBL_END}}
END_RESPONSE_TABLE;

//{{JPFlatComboBox Implementation}}


JPFlatComboBox::JPFlatComboBox(TWindow* parent, int id, int x, int y, int w, int h, uint32 style, uint textLimit, TModule* module)
:
    TComboBox(parent, id, x, y, w, h, style, textLimit, module)
{
  // INSERT>> Your constructor code here.
   MouseInControl = false;
   MaxListWidth = w;
   hwndEdit = NULL;
   IsInWin31 = false;
#if defined(BI_PLAT_WIN16)
   DWORD dwVersion = ::GetVersion();
   if ((LOBYTE(LOWORD(dwVersion)) == 3) && (HIBYTE(LOWORD(dwVersion)) < 95))
      IsInWin31 = true;
#endif
}


JPFlatComboBox::~JPFlatComboBox()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

}

void JPFlatComboBox::RedrawBorder(void)
{
  TWindowDC mydc(GetHandle());
  TPoint point;
  GetCursorPos(point);
  ScreenToClient(point);
  TRect rect;
  rect = GetClientRect();
  int thumbx = ::GetSystemMetrics(SM_CXHTHUMB);
  TRect crect;
  if ((IsInWin31) && (hwndEdit))
      {
      crect = rect;
      InflateRect(&crect, -2, -2);
      crect.right = crect.right - thumbx + 3;
      ::MoveWindow(hwndEdit, crect.left, crect.top, crect.right - crect.left,
                             crect.bottom - crect.top, false);
      ::UpdateWindow(hwndEdit);
      TBrush editbrush (TColor::SysWindow);
      mydc.FrameRect(crect, editbrush);
      mydc.RestoreBrush();
      }
  crect = rect;
  InflateRect(&crect, -3, -3);
  if (PtInRect(&crect, point))
   {
   TBrush brush (TColor::Sys3dFace);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush);
   rect = GetClientRect();
   ::InflateRect (&rect, -1, -1);
   TUIBorder border(rect, TUIBorder::TStyle(TUIBorder::Recessed));
   border.Paint(mydc);
   }
else
   {
   TBrush brush (TColor::Sys3dFace);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush);
   TBrush brush2 (TColor::SysWindow);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush2);
   if (IsInWin31)
      {
      /*if (hwndEdit)
         {
         TRect w31rect = rect;
         w31rect.top = w31rect.top - 1;
         w31rect.bottom = w31rect.bottom + 1;
         w31rect.right = w31rect.right - thumbx + 2;
         ::MoveWindow(hwndEdit, w31rect.left, w31rect.top, w31rect.right - w31rect.left,
                                w31rect.bottom - w31rect.top, true);
         }*/
      rect.left = rect.right - thumbx + 3;
      }
   else
      rect.left = rect.right - thumbx + 1;
   mydc.FrameRect(rect,brush2);
   }
}

void JPFlatComboBox::SetupWindow()
{
  TComboBox::SetupWindow();

  // INSERT>> Your code here.
#if defined(BI_PLAT_WIN32)
  int Height = GetItemHeight(-1) - 2;
//  SetItemHeight(-1, Height);
  SetItemHeight(-1, 15);
#endif
  if (((GetStyle() & CBS_DROPDOWN) == CBS_DROPDOWN))
     hwndEdit = GetWindow(GW_CHILD);
  if (IsInWin31)
     {
     int Height = GetItemHeight(-1) + 4;
     SetItemHeight(-1, Height);
     if (hwndEdit)
        InsertEditTooltip(hwndEdit);
     }
  else SetItemHeight(-1, 15);
/*  if (hwndEdit)
     {
     if (lpfnOldWndProc == NULL)
        {
        lpfnOldWndProc = (WNDPROC) ::SetWindowLong(GetHandle(), GWL_WNDPROC, (DWORD) SubClassProc);
        HWND appHwnd = GetApplication()->GetMainWindow()->GetHandle();
        }
     }*/
}


void JPFlatComboBox::EvKillFocus(THandle hWndGetFocus )
{
    TComboBox::EvKillFocus(hWndGetFocus );

    Invalidate(false);
}

void JPFlatComboBox::EvSetFocus(THandle hWndLostFocus )
{
    TComboBox::EvSetFocus(hWndLostFocus );

    Invalidate(false);
}

HBRUSH JPFlatComboBox::EvCtlColor(HDC hDC, THandle hWndChild, uint ctlType)
{
  HBRUSH result;

  // INSERT>> Your code here.
  if (ctlType == CTLCOLOR_LISTBOX)
   {
   RECT rect;
   ::GetWindowRect(hWndChild, &rect);
   if ((rect.right - rect.left)!=MaxListWidth)
      ::MoveWindow(hWndChild, rect.left, rect.top, MaxListWidth, rect.bottom - rect.top, true);
   }
  else if ((ctlType == CTLCOLOR_EDIT) && (IsInWin31))
         {
         TBrush brush (TColor::SysWindow);
         ::SetBkColor(hDC, TColor::SysWindow);
         return (HBRUSH) brush;
         }
  else if ((ctlType == CTLCOLOR_MSGBOX) && (IsInWin31))
         {
         TBrush brush (TColor::SysWindow);
         ::SetBkColor(hDC, TColor::SysWindow);
         return (HBRUSH) brush;
         }
  result = TComboBox::EvCtlColor(hDC, hWndChild, ctlType);
  return result;
}


void JPFlatComboBox::EvPaint()
{
    TComboBox::EvPaint();

  /*if (IsInWin31)
     {
     if (((GetStyle() & CBS_DROPDOWN) == CBS_DROPDOWN))
        {
        HWND hedit = GetWindow(GW_CHILD);
        if (hedit)
           ::UpdateWindow(hedit);
        }
     }*/
  // INSERT>> Your code here.
    RedrawBorder();
}


void JPFlatComboBox::EvLButtonDown(uint modKeys, TPoint& point)
{
    TComboBox::EvLButtonDown(modKeys, point);

  // INSERT>> Your code here.
    RedrawBorder();
}


void JPFlatComboBox::EvLButtonUp(uint modKeys, TPoint& point)
{
    TComboBox::EvLButtonUp(modKeys, point);

  // INSERT>> Your code here.
}

int JPFlatComboBox::SetListBoxWidth(uint newwidth)
{
  MaxListWidth = newwidth;
  return MaxListWidth;
}

uint JPFlatComboBox::GetListBoxWidth() const
{
  return (uint) MaxListWidth;
}


bool JPFlatComboBox::EvEraseBkgnd(HDC dc)
{
  bool result;

  result = TComboBox::EvEraseBkgnd(dc);

  // INSERT>> Your code here.
  if (IsInWin31)
     RedrawBorder();

  return result;
}

void JPFlatComboBox::InsertEditTooltip(HWND editHwnd)
{
  TWindow* tWin = GetParentO();
  if (tWin)
     {
     TGadgetWindow* gWin = TYPESAFE_DOWNCAST(tWin, TGadgetWindow);
     if (gWin)
        {
        TTooltip* tTip = gWin->GetTooltip();
        if (tTip)
           {
           TToolInfo toolInfo((HWND) gWin->GetHandle(), (HWND) GetHandle());
           if (tTip->GetToolInfo(toolInfo))
              {
              if (toolInfo.GetCacheText())
                 {
                 toolInfo.SetToolInfo(editHwnd, gWin->GetHandle());
                 toolInfo.SetText("Edit Control Tooltip");
                 tTip->AddTool(toolInfo);
                 }
              }
           }
        }
     }
}

uint JPFlatComboBox::EvNCHitTest(TPoint& point)
{
  uint result;

  result = TComboBox::EvNCHitTest(point);

  // INSERT>> Your code here.
  ScreenToClient(point);
  TRect rect;
  rect = GetClientRect();
  InflateRect(&rect, -3, -3);
  if (PtInRect(&rect, point))
     {
     InflateRect(&rect, 3, 3);
     MouseInControl = true;
     int thumbx = ::GetSystemMetrics(SM_CXHTHUMB);
     if (IsInWin31)
         rect.left = rect.right - thumbx + 3;
     else
         rect.left = rect.right - thumbx - 3;
//     if ((IsInWin31) || (PtInRect(&rect, point)))
        RedrawBorder();
//     else
//        Invalidate(false);
     }
  else
  		{
      MouseInControl = false;
      RedrawBorder();
      }

  return result;
}


bool JPFlatComboBox::IdleAction(long idleCount)
{
  // INSERT>> Your code here.
  if (idleCount == 0)
     {
     TPoint point;
     GetCursorPos(point);
     ScreenToClient(point);
     TRect rect = GetClientRect();
     InflateRect(&rect, -3, -3);
     if (PtInRect(&rect, point))
        {
        if (MouseInControl == false)
           {
           MouseInControl = true;
           //Invalidate(false);
           RedrawBorder();
           }
        }
     else
        {
        if (MouseInControl == true)
           {
           MouseInControl = false;
           //Invalidate(false);
           RedrawBorder();
           }
        }
     }
  return false;
}


void JPFlatComboBox::EvMouseMove(uint modKeys, TPoint& point)
{
    TComboBox::EvMouseMove(modKeys, point);

  // INSERT>> Your code here.
  //ScreenToClient(point);
  TRect rect;
  rect = GetClientRect();
  InflateRect(&rect, -3, -3);
  if ((PtInRect(&rect, point)) && (MouseInControl == false))
     {
     MouseInControl = true;
     RedrawBorder();
     }
  else
  		{
      MouseInControl = false;
      RedrawBorder();
      }
}

TResult
JPFlatComboBox::ReceiveMessage(uint msg, TParam1 param1, TParam2 param2)
{
 TResult result = TComboBox::ReceiveMessage(msg, param1, param2);

 HWND hwnd = GetApplication()->GetMainWindow()->GetHandle();

 if (hwnd)
    ::SendMessage(hwnd, msg, param1, param2);

 return result;
}

/*TResult CALLBACK SubClassProc(HWND hwnd, UINT msg, TParam1 wParam, TParam2 lParam)
{
    if (msg == WM_NOTIFY)
       ::SendMessage(appHwnd, msg, wParam, lParam);
    return ::CallWindowProc(lpfnOldWndProc, hwnd, msg, wParam, lParam);
}*/

