/*************************************************************************************
*  Copyright (c) 1997-1998 by Joseph Parrello. All Rights Reserved.                  *
*  Site   : http://space.tin.it/computer/giparrel                                    *
*  E-Mail : joparrel@tin.it, giusparr@tin.it, giparre@tin.it                         *
*                                                                                    *
*  Revision:   1.0  (May 15, 1998).                                                  *
*                                                                                    *
*  Definition of JPFlatDivGadget class.                                              *
*                                                                                    *
*  Read JPFLTDIV.TXT for use and license.                                            *
*                                                                                    *
*  Only for OWL 5.02 and 5.01 users.                                                 *
*  OWL and ObjectWindows are trademarks of Borland International.                    *
*                                                                                    *
*  THIS CLASS AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF                *
*  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO               *
*  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                    *
*  PARTICULAR PURPOSE.                                                               *
*************************************************************************************/
#include <owl/pch.h>

#if !defined(OWL_CELARRAY_H)
# include <owl/celarray.h>
#endif
#if !defined(OWL_GADGETWI_H)
# include <owl/gadgetwi.h>
#endif

#include "jpfltdiv.h"


//{{JPFlatDivGadget Implementation}}


JPFlatDivGadget::JPFlatDivGadget(int size, TGadget* ref, bool EndOfLine, bool NoVertBar)
:
    TGadget(0, None)
{
  // INSERT>> Your constructor code here.
  NoDrawOnVertBar   = NoVertBar;
  DrawAtEndOfLine   = EndOfLine;
  ShrinkWrapWidth   = false;   //true
  ShrinkWrapHeight  = false;  //true
  TBorders borders;
  borders.Top = borders.Left = borders.Right = borders.Bottom = 1;
  SetBorders(borders);
  DefaultSize = 6;
  if ((!size) || (size < DefaultSize))
    size = DefaultSize;
  divSize = size;
  gadgRef = ref;
}


JPFlatDivGadget::~JPFlatDivGadget()
{
  // INSERT>> Your destructor code here.

}


void JPFlatDivGadget::GetDesiredSize(TSize& size)
{
  TGadget::GetDesiredSize(size);

  // INSERT>> Your code here.
  if (Window)
     {
     TSize maxsize(0, 0);
     if ((gadgRef) && (gadgRef->GetId() != 0))   // follows another gadget if it is specified
        {
        gadgRef->GetDesiredSize(maxsize);
        if ((maxsize.cx > 0) || (maxsize.cy > 0))
           size = maxsize;
        }
     else
        {
        for (TGadget* gadg = Window->FirstGadget(); gadg; gadg = Window->NextGadget(*gadg))
           {
           JPFlatDivGadget* bg = TYPESAFE_DOWNCAST(gadg, JPFlatDivGadget);
           if (!bg)    // avoids an endless loop
              {
              TSize gsize(0, 0);
              gadg->GetDesiredSize(gsize);
              if (gsize.cx > maxsize.cx)
                 maxsize.cx = gsize.cx;
              if (gsize.cy > maxsize.cy)
                 maxsize.cy = gsize.cy;
              }
           }
        if ((maxsize.cx > 0) || (maxsize.cy > 0))
           size = maxsize;
        }
     if (Window->GetDirection()==TGadgetWindow::Horizontal)
           size.cx = divSize;
     else
     if (Window->GetDirection()==TGadgetWindow::Vertical)
           size.cy = divSize;
     else
     if (Window->GetDirection()==TGadgetWindow::Rectangular)
           size.cx = divSize;
     }
}

void JPFlatDivGadget::DrawDividerLine(TDC& dc, int x1, int y1, int x2, int y2, bool isFirst)
{
     TPen pen (isFirst ? TColor::Sys3dShadow : TColor::Sys3dHilight);
     dc.SelectObject(pen);
     dc.MoveTo(x1, y1);
     dc.LineTo(x2, y2);
     dc.RestorePen();
}

void JPFlatDivGadget::Paint(TDC& dc)
{
  // INSERT>> Your code here.
  TRect boundsRect;

  int cx = Bounds.Width();
  int cy = Bounds.Height();
  if (cx > cy)
        {
        if (NoDrawOnVertBar == false)
           {
           int y1 = (cy - 2) / 2;
           DrawDividerLine(dc, 0, y1, cx, y1, true);
           DrawDividerLine(dc, 0, y1 + 1, cx, y1 + 1, false);
           }
        }
  else
     if (((IsEndOfRow() == false) || (DrawAtEndOfLine)) || (Window->GetDirection() != TGadgetWindow::Rectangular))
        {
        int x1 = (cx - 2) / 2;
        DrawDividerLine(dc, x1, 0, x1, cy, true);
        DrawDividerLine(dc, x1 + 1, 0, x1 + 1, cy, false);
        }
}

void JPFlatDivGadget::SetEndOfLineStatus(bool EndOfLine)
{
    if (EndOfLine != DrawAtEndOfLine)
       {
       DrawAtEndOfLine = EndOfLine;
       if (IsEndOfRow())
          Invalidate();   // update the gadget only if at the end of the row
       }
}

bool JPFlatDivGadget::GetEndOfLineStatus() const
{
    return DrawAtEndOfLine;
}

void JPFlatDivGadget::SetNoVertBarStatus(bool NoVertBar)
{
    if (NoVertBar != NoDrawOnVertBar)
       {
       NoDrawOnVertBar = NoVertBar;
       Invalidate();
       }
}

bool JPFlatDivGadget::GetNoVertBarStatus() const
{
    return NoDrawOnVertBar;
}

void JPFlatDivGadget::SetReferenceGadget(TGadget* ref)
{
    if (ref != gadgRef)
       {
       gadgRef = ref;
       if (Window)
          Window->GadgetChangedSize(*this);
       Invalidate();
       }
}

TGadget* JPFlatDivGadget::GetReferenceGadget(void)
{
    return gadgRef;
}

int JPFlatDivGadget::GetDividerSize() const
{
    return divSize;
}

int JPFlatDivGadget::GetDivDefaultSize() const
{
    return DefaultSize;
}

bool JPFlatDivGadget::IsDefaultSize() const
{
    return (GetDividerSize() == DefaultSize);
}

